@@ -55,6 +55,7 @@ CONTEXT_VARIABLES = [
     "model",
     "monitor",
     "netdevice",
+    "no_kvm",
     "serial",
     "vga",
     # u-boot variables
@@ -71,6 +72,7 @@ CONTEXT_VARIABLES = [
     "boot_character_delay",
     "bootloader_prompt",
     "console_device",
+    "custom_kernel_args",
     "extra_kernel_args",
     "extra_nfsroot_args",
     "kernel_loglevel",
@@ -93,7 +95,8 @@ def validate_action(name, index, data, strict=True):
     except ImportError:
         raise Invalid("unknown action type", path=["actions"] + name.split("."))
     except MultipleInvalid as exc:
-        raise Invalid(exc.msg, path=["actions[%d]" % index] + name.split(".")) from exc
+        path = ["actions[%d]" % index] + name.split(".") + exc.path
+        raise Invalid(exc.msg, path=path) from exc
 
 
 def validate(data, strict=True, extra_context_variables=[]):
@@ -112,7 +115,10 @@ def validate(data, strict=True, extra_context_variables=[]):
             cls = "deploy." + data.get("to", "")
         elif action_type == "test":
             if "definitions" in data:
-                cls = "test.definition"
+                if "docker" in data:
+                    cls = "test.docker"
+                else:
+                    cls = "test.definition"
             elif "interactive" in data:
                 cls = "test.interactive"
             elif "monitors" in data:
@@ -192,6 +198,7 @@ def notify():
 def extra_checks(data):
     check_job_timeouts(data)
     check_multinode_or_device_type(data)
+    check_multinode_or_environment(data)
     check_multinode_roles(data)
     check_namespace(data)
     check_secrets_visibility(data)
@@ -242,6 +249,14 @@ def check_multinode_or_device_type(data):
         raise Invalid('"device_type" or multinode should be defined')
 
 
+def check_multinode_or_environment(data):
+    environment = data.get("environment")
+    multinode = data.get("protocols", {}).get("lava-multinode")
+
+    if environment and multinode:
+        raise Invalid('"environment" shoud not be used with multinode')
+
+
 def check_multinode_roles(data):
     # When using multinode, each action should have a role
     if not data.get("protocols", {}).get("lava-multinode"):
@@ -305,6 +320,7 @@ def job(extra_context_variables=[]):
             Optional("priority"): Any("high", "medium", "low", Range(min=0, max=100)),
             Optional("tags"): [str],
             Optional("secrets"): dict,
+            Optional("environment"): dict,
             Optional("protocols"): {
                 Optional("lava-lxc"): Any(lava_lxc, {str: lava_lxc}),
                 Optional("lava-multinode"): {
@@ -318,6 +334,7 @@ def job(extra_context_variables=[]):
                                     extra=False,
                                 ),
                                 Optional("tags"): [str],
+                                Optional("environment"): dict,
                                 Optional("timeout"): timeout(),
                             },
                             {
