@@ -25,14 +25,35 @@ from voluptuous import Any, Optional, Required
 from lava_common.schemas import action
 
 
-def url():
-    return {
+def url(extra=None):
+    if extra is None:
+        extra = {}
+
+    base_url = {
         Required("url"): str,
-        Optional("compression"): Any("bz2", "gz", "xz", "zip", None),
+        Optional("compression"): Any("bz2", "gz", "xz", "zip", "zstd", None),
         Optional("archive"): "tar",
         Optional("md5sum"): str,
         Optional("sha256sum"): str,
+        Optional("sha512sum"): str,
     }
+    return Any(
+        {**base_url, **extra},
+        {
+            **base_url,
+            **extra,
+            Required("format"): Any("cpio.newc", "ext4"),
+            Optional("partition"): int,
+            Required("overlays"): {
+                Optional("lava"): bool,
+                str: {
+                    **base_url,
+                    Required("format"): Any("tar"),
+                    Required("path"): str,
+                },
+            },
+        },
+    )
 
 
 def schema():
