@@ -18,20 +18,20 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
+import shlex
 
-import os
 from lava_dispatcher.action import Action, Pipeline
 from lava_common.exceptions import InfrastructureError, JobError, LAVABug
-from lava_dispatcher.logical import Boot
+from lava_dispatcher.logical import Boot, RetryAction
 from lava_dispatcher.actions.boot import (
-    BootAction,
-    AutoLoginAction,
-    OverlayUnpack,
     AdbOverlayUnpack,
+    AutoLoginAction,
+    BootHasMixin,
+    OverlayUnpack,
 )
 from lava_dispatcher.power import ResetDevice, PreOs
-from lava_common.constants import LAVA_LXC_HOME
-from lava_dispatcher.utils.lxc import is_lxc_requested, lxc_cmd_prefix
+from lava_dispatcher.utils.fastboot import BaseAction
+from lava_dispatcher.utils.udev import WaitDeviceBoardID
 from lava_dispatcher.connections.serial import ConnectDevice
 from lava_dispatcher.connections.adb import ConnectAdb
 from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
@@ -60,12 +60,9 @@ class BootFastboot(Boot):
 
     compatibility = 1
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = BootFastbootAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return BootFastbootAction()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -75,7 +72,7 @@ class BootFastboot(Boot):
         return False, 'boot "method" was not "fastboot"'
 
 
-class BootFastbootCommands(Action):
+class BootFastbootCommands(BaseAction):
 
     name = "fastboot-boot-commands"
     description = "Run custom fastboot commands before boot"
@@ -85,15 +82,10 @@ class BootFastbootCommands(Action):
         serial_number = self.job.device["fastboot_serial_number"]
         self.logger.info("Running custom fastboot boot commands....")
         for command in self.parameters.get("commands"):
-            pre_cmd = (
-                lxc_cmd_prefix(self.job)
-                + ["fastboot", "-s", serial_number, command]
-                + self.job.device["fastboot_options"]
-            )
-            self.run_command(pre_cmd)
+            self.run_fastboot(shlex.split(command))
 
 
-class BootFastbootAction(BootAction):
+class BootFastbootAction(BootHasMixin, RetryAction, BaseAction):
     """
     Provide for auto_login parameters in this boot stanza and re-establish the
     connection after boot.
@@ -114,49 +106,52 @@ class BootFastbootAction(BootAction):
             self.errors = "fastboot_sequence undefined"
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.parameters = parameters
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
 
         if parameters.get("commands"):
-            self.internal_pipeline.add_action(BootFastbootCommands())
+            self.pipeline.add_action(BootFastbootCommands())
+
+        board_id = self.job.device["fastboot_serial_number"]
 
         # Always ensure the device is in fastboot mode before trying to boot.
         # Check if the device has a power command such as HiKey, Dragonboard,
         # etc. against device that doesn't like Nexus, etc.
         if self.job.device.get("fastboot_via_uboot", False):
-            self.internal_pipeline.add_action(ConnectDevice())
-            self.internal_pipeline.add_action(UBootEnterFastbootAction())
+            self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(UBootEnterFastbootAction())
         elif self.job.device.hard_reset_command:
             self.force_prompt = True
-            self.internal_pipeline.add_action(ConnectDevice())
-            self.internal_pipeline.add_action(ResetDevice())
+            self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(ResetDevice())
         else:
-            self.internal_pipeline.add_action(EnterFastbootAction())
+            self.pipeline.add_action(WaitDeviceBoardID(board_id))
+            self.pipeline.add_action(EnterFastbootAction())
 
         # Based on the boot sequence defined in the device configuration, add
         # the required pipeline actions.
         sequences = self.job.device["actions"]["boot"]["methods"].get("fastboot", [])
         for sequence in sequences:
             mapped = _fastboot_sequence_map(sequence)
+            self.pipeline.add_action(WaitDeviceBoardID(board_id))
             if mapped[1]:
-                self.internal_pipeline.add_action(mapped[0](device_actions=mapped[1]))
+                self.pipeline.add_action(mapped[0](device_actions=mapped[1]))
             elif mapped[0]:
-                self.internal_pipeline.add_action(mapped[0]())
+                self.pipeline.add_action(mapped[0]())
         if self.job.device.hard_reset_command:
-            if not is_lxc_requested(self.job):
-                self.internal_pipeline.add_action(PreOs())
+            if not self.is_container():
+                self.pipeline.add_action(PreOs())
             if self.has_prompts(parameters):
-                self.internal_pipeline.add_action(AutoLoginAction())
+                self.pipeline.add_action(AutoLoginAction())
                 if self.test_has_shell(parameters):
-                    self.internal_pipeline.add_action(ExpectShellSession())
+                    self.pipeline.add_action(ExpectShellSession())
                     if "transfer_overlay" in parameters:
-                        self.internal_pipeline.add_action(OverlayUnpack())
-                    self.internal_pipeline.add_action(ExportDeviceEnvironment())
+                        self.pipeline.add_action(OverlayUnpack())
+                    self.pipeline.add_action(ExportDeviceEnvironment())
         else:
-            if not is_lxc_requested(self.job):
-                self.internal_pipeline.add_action(ConnectAdb())
-                self.internal_pipeline.add_action(AdbOverlayUnpack())
+            if not self.is_container():
+                self.pipeline.add_action(ConnectAdb())
+                self.pipeline.add_action(AdbOverlayUnpack())
 
 
 class WaitFastBootInterrupt(Action):
@@ -206,7 +201,7 @@ class WaitFastBootInterrupt(Action):
         return connection
 
 
-class FastbootBootAction(Action):
+class FastbootBootAction(BaseAction):
     """
     This action calls fastboot to boot into the system.
     """
@@ -228,22 +223,13 @@ class FastbootBootAction(Action):
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
-        lxc_name = is_lxc_requested(self.job)
-        serial_number = self.job.device["fastboot_serial_number"]
+
         boot_img = self.get_namespace_data(
             action="download-action", label="boot", key="file"
         )
-        if not boot_img:
-            raise JobError("Boot image not found, unable to boot")
-        else:
-            if lxc_name:
-                boot_img = os.path.join(LAVA_LXC_HOME, os.path.basename(boot_img))
-        fastboot_cmd = (
-            lxc_cmd_prefix(self.job)
-            + ["fastboot", "-s", serial_number, "boot", boot_img]
-            + self.job.device["fastboot_options"]
-        )
-        command_output = self.parsed_command(fastboot_cmd, allow_fail=True)
+        boot_img = self.maybe_copy_to_container(boot_img)
+
+        command_output = self.get_fastboot_output(["boot", boot_img], allow_fail=True)
         if command_output and "booting" not in command_output.lower():
             raise JobError("Unable to boot with fastboot: %s" % command_output)
         else:
@@ -259,7 +245,7 @@ class FastbootBootAction(Action):
         return connection
 
 
-class FastbootRebootAction(Action):
+class FastbootRebootAction(BaseAction):
     """
     This action calls fastboot to reboot into the system.
     """
@@ -281,14 +267,7 @@ class FastbootRebootAction(Action):
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
-        serial_number = self.job.device["fastboot_serial_number"]
-        fastboot_opts = self.job.device["fastboot_options"]
-        fastboot_cmd = (
-            lxc_cmd_prefix(self.job)
-            + ["fastboot", "-s", serial_number, "reboot"]
-            + fastboot_opts
-        )
-        command_output = self.parsed_command(fastboot_cmd, allow_fail=True)
+        command_output = self.get_fastboot_output(["reboot"], allow_fail=True)
         if command_output and "rebooting" not in command_output.lower():
             raise JobError("Unable to fastboot reboot: %s" % command_output)
         else:
@@ -304,7 +283,7 @@ class FastbootRebootAction(Action):
         return connection
 
 
-class EnterFastbootAction(Action):
+class EnterFastbootAction(BaseAction):
     """
     Enters fastboot bootloader.
     """
@@ -332,39 +311,26 @@ class EnterFastbootAction(Action):
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
 
-        cmd_prefix = lxc_cmd_prefix(self.job)
         # Try to enter fastboot mode with adb.
         adb_serial_number = self.job.device["adb_serial_number"]
         # start the adb daemon
-        adb_cmd = cmd_prefix + ["adb", "start-server"]
-        command_output = self.parsed_command(adb_cmd, allow_fail=True)
+        command_output = self.get_adb_output(["start-server"], allow_fail=True)
         if command_output and "successfully" in command_output:
             self.logger.debug("adb daemon started: %s", command_output)
-        adb_cmd = cmd_prefix + ["adb", "-s", adb_serial_number, "devices"]
-        command_output = self.parsed_command(adb_cmd, allow_fail=True)
+
+        command_output = self.get_adb_output(["devices"], allow_fail=True)
         if command_output and adb_serial_number in command_output:
             self.logger.debug("Device is in adb: %s", command_output)
-            adb_cmd = cmd_prefix + ["adb", "-s", adb_serial_number, "reboot-bootloader"]
-            self.run_command(adb_cmd)
+            self.run_adb(["reboot-bootloader"])
             return connection
 
         # Enter fastboot mode with fastboot.
         fastboot_serial_number = self.job.device["fastboot_serial_number"]
-        fastboot_opts = self.job.device["fastboot_options"]
-        fastboot_cmd = (
-            cmd_prefix
-            + ["fastboot", "-s", fastboot_serial_number, "devices"]
-            + fastboot_opts
-        )
-        command_output = self.parsed_command(fastboot_cmd)
+        command_output = self.get_fastboot_output(["devices"])
+
         if command_output and fastboot_serial_number in command_output:
             self.logger.debug("Device is in fastboot: %s", command_output)
-            fastboot_cmd = (
-                cmd_prefix
-                + ["fastboot", "-s", fastboot_serial_number, "reboot-bootloader"]
-                + fastboot_opts
-            )
-            command_output = self.parsed_command(fastboot_cmd)
+            command_output = self.get_fastboot_output(["reboot-bootloader"])
             if command_output and "okay" not in command_output.lower():
                 raise InfrastructureError(
                     "Unable to enter fastboot: %s" % command_output
