@@ -18,6 +18,7 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
+import guestfs
 import os
 import shutil
 import subprocess  # nosec - internal use.
@@ -26,7 +27,6 @@ from lava_common.exceptions import InfrastructureError, JobError, LAVABug
 from lava_common.constants import RAMDISK_FNAME, UBOOT_DEFAULT_HEADER_LENGTH
 from lava_common.utils import debian_filename_version
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
-from lava_dispatcher.utils.contextmanager import chdir
 from lava_dispatcher.utils.installers import add_late_command, add_to_kickstart
 from lava_dispatcher.utils.filesystem import (
     lxc_path,
@@ -37,19 +37,24 @@ from lava_dispatcher.utils.filesystem import (
     copy_overlay_to_sparse_fs,
 )
 from lava_dispatcher.utils.shell import which
-from lava_dispatcher.utils.compression import compress_file, decompress_file, untar_file
+from lava_dispatcher.utils.compression import (
+    compress_file,
+    cpio,
+    decompress_file,
+    untar_file,
+    uncpio,
+)
 from lava_dispatcher.utils.strings import substitute
 from lava_dispatcher.utils.network import dispatcher_ip
 from lava_dispatcher.actions.deploy.prepare import PrepareKernelAction
 
-# pylint: disable=superfluous-parens,too-many-statements
-
 
 class ApplyOverlayGuest(Action):
 
     name = "apply-overlay-guest"
     description = "prepare a qcow2 drive containing the overlay"
     summary = "build a guest filesystem with the overlay"
+    timeout_exception = InfrastructureError
 
     def __init__(self):
         super().__init__()
@@ -101,6 +106,7 @@ class ApplyOverlayImage(Action):
     name = "apply-overlay-image"
     description = "apply overlay via guestfs to the test image"
     summary = "apply overlay to test image"
+    timeout_exception = InfrastructureError
 
     def __init__(self, image_key="image", use_root_partition=True):
         super().__init__()
@@ -120,7 +126,25 @@ class ApplyOverlayImage(Action):
             root_partition = None
 
             if self.use_root_partition:
-                root_partition = self.parameters[self.image_key].get("root_partition")
+                if (
+                    self.image_key not in self.parameters.keys()
+                    and "images" in self.parameters.keys()
+                ):
+                    if self.image_key in self.parameters["images"]:
+                        root_partition = self.parameters["images"][self.image_key].get(
+                            "root_partition"
+                        )
+                    else:
+                        raise JobError(
+                            "Unable to find image configuration for '{image}'".format(
+                                image=self.image_key
+                            )
+                        )
+                else:
+                    root_partition = self.parameters[self.image_key].get(
+                        "root_partition"
+                    )
+
                 if root_partition is None:
                     raise JobError(
                         "Unable to apply the overlay image without 'root_partition'"
@@ -138,6 +162,8 @@ class ApplyOverlaySparseImage(Action):
     name = "apply-overlay-sparse-image"
     description = "apply overlay to sparse image"
     summary = "apply overlay to sparse image"
+    command_exception = InfrastructureError
+    timeout_exception = InfrastructureError
 
     def __init__(self, image_key):
         super().__init__()
@@ -146,11 +172,11 @@ class ApplyOverlaySparseImage(Action):
     def validate(self):
         super().validate()
         binary = which("simg2img")
-        self.logger.info(debian_filename_version(binary, label=True))
+        self.logger.info(debian_filename_version(binary))
         which("mount")
         which("umount")
         binary = which("img2simg")
-        self.logger.info(debian_filename_version(binary, label=True))
+        self.logger.info(debian_filename_version(binary))
 
     def run(self, connection, max_end_time):
         overlay_file = self.get_namespace_data(
@@ -190,34 +216,31 @@ class PrepareOverlayTftp(Action):
     name = "prepare-tftp-overlay"
     description = "extract ramdisk or nfsrootfs in preparation for lava overlay"
     summary = "extract ramdisk or nfsrootfs"
+    timeout_exception = InfrastructureError
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        self.internal_pipeline.add_action(
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.pipeline.add_action(
             ExtractNfsRootfs()
         )  # idempotent, checks for nfsrootfs parameter
-        self.internal_pipeline.add_action(
-            OverlayAction()
-        )  # idempotent, includes testdef
-        self.internal_pipeline.add_action(
+        self.pipeline.add_action(OverlayAction())  # idempotent, includes testdef
+        self.pipeline.add_action(
             ExtractRamdisk()
         )  # idempotent, checks for a ramdisk parameter
-        self.internal_pipeline.add_action(
+        self.pipeline.add_action(
             ExtractModules()
         )  # idempotent, checks for a modules parameter
-        self.internal_pipeline.add_action(ApplyOverlayTftp())
+        self.pipeline.add_action(ApplyOverlayTftp())
         if "kernel" in parameters and "type" in parameters["kernel"]:
-            self.internal_pipeline.add_action(PrepareKernelAction())
-        self.internal_pipeline.add_action(
+            self.pipeline.add_action(PrepareKernelAction())
+        self.pipeline.add_action(
             ConfigurePreseedFile()
         )  # idempotent, checks for a preseed parameter
-        self.internal_pipeline.add_action(
+        self.pipeline.add_action(
             CompressRamdisk()
         )  # idempotent, checks for a ramdisk parameter
         if "depthcharge" in self.job.device["actions"]["boot"]["methods"]:
-            self.internal_pipeline.add_action(PrepareKernelAction())
+            self.pipeline.add_action(PrepareKernelAction())
 
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
@@ -241,6 +264,7 @@ class ApplyOverlayTftp(Action):
     name = "apply-overlay-tftp"
     description = "unpack the overlay into the nfsrootfs or ramdisk"
     summary = "apply lava overlay test files"
+    timeout_exception = InfrastructureError
 
     def validate(self):
         super().validate()
@@ -251,7 +275,7 @@ class ApplyOverlayTftp(Action):
             if "address" not in persist:
                 self.errors = "Missing address for persistent NFS"
 
-    def run(self, connection, max_end_time):  # pylint: disable=too-many-branches
+    def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         directory = None
         nfs_address = None
@@ -332,7 +356,6 @@ class ApplyOverlayTftp(Action):
             )
         else:
             self.logger.debug("[%s] No overlay directory", namespace)
-            self.logger.debug(self.parameters)
         if self.parameters.get("os") == "centos_installer":
             # centos installer ramdisk doesnt like having anything other
             # than the kickstart config being inserted. Instead, make the
@@ -368,7 +391,7 @@ class ApplyOverlayTftp(Action):
         return connection
 
 
-class ExtractRootfs(Action):  # pylint: disable=too-many-instance-attributes
+class ExtractRootfs(Action):
     """
     Unpacks the rootfs and applies the overlay to it
     """
@@ -376,6 +399,7 @@ class ExtractRootfs(Action):  # pylint: disable=too-many-instance-attributes
     name = "extract-rootfs"
     description = "unpack rootfs"
     summary = "unpack rootfs, ready to apply lava overlay"
+    timeout_exception = InfrastructureError
 
     def __init__(self):
         super().__init__()
@@ -409,6 +433,7 @@ class ExtractNfsRootfs(ExtractRootfs):
     name = "extract-nfsrootfs"
     description = "unpack nfsrootfs"
     summary = "unpack nfsrootfs, ready to apply lava overlay"
+    timeout_exception = InfrastructureError
 
     def __init__(self):
         super().__init__()
@@ -423,10 +448,6 @@ class ExtractNfsRootfs(ExtractRootfs):
             action="download-action", label=self.param_key, key="file"
         ):
             self.errors = "no file specified extract as %s" % self.param_key
-        if not os.path.exists("/usr/sbin/exportfs"):
-            raise InfrastructureError(
-                "NFS job requested but nfs-kernel-server not installed."
-            )
         if "prefix" in self.parameters[self.param_key]:
             prefix = self.parameters[self.param_key]["prefix"]
             if prefix.startswith("/"):
@@ -455,6 +476,14 @@ class ExtractNfsRootfs(ExtractRootfs):
             self.set_namespace_data(
                 action="extract-rootfs", label="file", key=self.file_key, value=root_dir
             )
+
+        self.job.device["dynamic_data"]["NFS_ROOTFS"] = self.get_namespace_data(
+            action="extract-rootfs", label="file", key=self.file_key
+        )
+        self.job.device["dynamic_data"]["NFS_SERVER_IP"] = dispatcher_ip(
+            self.job.parameters["dispatcher"], "nfs"
+        )
+
         return connection
 
 
@@ -467,6 +496,7 @@ class ExtractModules(Action):
     name = "extract-modules"
     description = "extract supplied kernel modules"
     summary = "extract kernel modules"
+    timeout_exception = InfrastructureError
 
     def run(self, connection, max_end_time):
         if not self.parameters.get("modules"):  # idempotency
@@ -478,35 +508,30 @@ class ExtractModules(Action):
         if not self.parameters.get("ramdisk"):
             if not self.parameters.get("nfsrootfs"):
                 raise JobError("Unable to identify a location for the unpacked modules")
+
         # if both NFS and ramdisk are specified, apply modules to both
         # as the kernel may need some modules to raise the network and
         # will need other modules to support operations within the NFS
         if self.parameters.get("nfsrootfs"):
             if not self.parameters["nfsrootfs"].get("install_modules", True):
                 self.logger.info("Skipping applying overlay to NFS")
-                return connection
-            root = self.get_namespace_data(
-                action="extract-rootfs", label="file", key="nfsroot"
-            )
-            self.logger.info("extracting modules file %s to %s", modules, root)
-            untar_file(modules, root)
+            else:
+                root = self.get_namespace_data(
+                    action="extract-rootfs", label="file", key="nfsroot"
+                )
+                self.logger.info("extracting modules file %s to %s", modules, root)
+                untar_file(modules, root)
         if self.parameters.get("ramdisk"):
             if not self.parameters["ramdisk"].get("install_modules", True):
                 self.logger.info("Not adding modules to the ramdisk.")
-                return connection
-            root = self.get_namespace_data(
-                action="extract-overlay-ramdisk",
-                label="extracted_ramdisk",
-                key="directory",
-            )
-            self.logger.info("extracting modules file %s to %s", modules, root)
-            untar_file(modules, root)
-        try:
-            os.unlink(modules)
-        except OSError as exc:
-            raise InfrastructureError(
-                "Unable to remove tarball: '%s' - %s" % (modules, exc)
-            )
+            else:
+                root = self.get_namespace_data(
+                    action="extract-overlay-ramdisk",
+                    label="extracted_ramdisk",
+                    key="directory",
+                )
+                self.logger.info("extracting modules file %s to %s", modules, root)
+                untar_file(modules, root)
         return connection
 
 
@@ -522,6 +547,7 @@ class ExtractRamdisk(Action):
     name = "extract-overlay-ramdisk"
     description = "extract ramdisk to a temporary directory"
     summary = "extract the ramdisk"
+    timeout_exception = InfrastructureError
 
     def __init__(self):
         super().__init__()
@@ -575,14 +601,7 @@ class ExtractRamdisk(Action):
             # give the file a predictable name
             shutil.move(ramdisk, ramdisk_compressed_data)
         ramdisk_data = decompress_file(ramdisk_compressed_data, compression)
-
-        with chdir(extracted_ramdisk):
-            cmd = ("cpio -iud -F %s" % ramdisk_data).split(" ")
-            if not self.run_command(cmd):
-                raise JobError(
-                    "Unable to extract cpio archive: %s - missing header definition (i.e. u-boot)?"
-                    % ramdisk_data
-                )
+        uncpio(ramdisk_data, extracted_ramdisk)
 
         # tell other actions where the unpacked ramdisk can be found
         self.set_namespace_data(
@@ -605,6 +624,7 @@ class CompressRamdisk(Action):
     name = "compress-ramdisk"
     description = "recreate a ramdisk with the overlay applied."
     summary = "compress ramdisk with overlay"
+    timeout_exception = InfrastructureError
 
     def __init__(self):
         super().__init__()
@@ -640,7 +660,7 @@ class CompressRamdisk(Action):
                 else:
                     self.errors = "ramdisk: add_header: unknown header type"
 
-    def run(self, connection, max_end_time):  # pylint: disable=too-many-locals
+    def run(self, connection, max_end_time):
         if not self.parameters.get("ramdisk"):  # idempotency
             return connection
         if self.skip:
@@ -674,24 +694,12 @@ class CompressRamdisk(Action):
                     action=self.name, label="file", key="preseed_local", value=filename
                 )
 
-        with chdir(ramdisk_dir):
-            self.logger.info(
-                "Building ramdisk %s containing %s", ramdisk_data, ramdisk_dir
-            )
-            cmd = "find . | cpio --create --format='newc' > %s" % ramdisk_data
-            try:
-                log = subprocess.check_output(  # nosec - safe to use shell=True here, no external arguments
-                    cmd, shell=True, stderr=subprocess.STDOUT
-                )
-                log = log.decode("utf-8", errors="replace")
-            except OSError as exc:
-                raise InfrastructureError("Unable to create cpio filesystem: %s" % exc)
-            # lazy-logging would mean that the quoting of cmd causes invalid YAML
-            self.logger.debug("%s\n%s" % (cmd, log))  # pylint: disable=logging-not-lazy
+        self.logger.info("Building ramdisk %s containing %s", ramdisk_data, ramdisk_dir)
+        self.logger.debug(">> %s", cpio(ramdisk_dir, ramdisk_data))
 
-            # we need to compress the ramdisk with the same method is was submitted with
-            compression = self.parameters["ramdisk"].get("compression")
-            final_file = compress_file(ramdisk_data, compression)
+        # we need to compress the ramdisk with the same method is was submitted with
+        compression = self.parameters["ramdisk"].get("compression")
+        final_file = compress_file(ramdisk_data, compression)
 
         tftp_dir = os.path.dirname(
             self.get_namespace_data(
@@ -741,6 +749,7 @@ class ApplyLxcOverlay(Action):
     name = "apply-lxc-overlay"
     description = "apply the overlay to the container by copying"
     summary = "apply overlay on the container"
+    timeout_exception = InfrastructureError
 
     def __init__(self):
         super().__init__()
@@ -803,6 +812,7 @@ class ConfigurePreseedFile(Action):
     name = "configure-preseed-file"
     description = "add commands to automated installers, to copy the lava test overlay to the installed system"
     summary = "add commands to installer config"
+    timeout_exception = InfrastructureError
 
     def run(self, connection, max_end_time):
         if "deployment_data" not in self.parameters:
@@ -816,7 +826,7 @@ class ConfigurePreseedFile(Action):
                     self.parameters["deployment_data"]["installer_extra_cmd"],
                 )
             if self.parameters.get("os") == "centos_installer":
-                ip_addr = dispatcher_ip(self.job.parameters["dispatcher"])
+                ip_addr = dispatcher_ip(self.job.parameters["dispatcher"], "tftp")
                 overlay = self.get_namespace_data(
                     action="download-action", label="file", key="overlay"
                 )
@@ -832,3 +842,150 @@ class ConfigurePreseedFile(Action):
                     post_command[0],
                 )
         return connection
+
+
+class AppendOverlays(Action):
+
+    name = "append-overlays"
+    description = "append overlays to an image"
+    summary = "append overlays to an image"
+
+    # TODO: list libguestfs supported formats
+    IMAGE_FORMATS = ["cpio.newc", "ext4"]
+    OVERLAY_FORMATS = ["tar"]
+
+    def __init__(self, key, params):
+        super().__init__()
+        self.key = key
+        self.params = params
+
+    def validate(self):
+        super().validate()
+        # Check that we have "overlays" dict
+        if "overlays" not in self.params:
+            raise JobError("Missing 'overlays' dictionary")
+        if not isinstance(self.params["overlays"], dict):
+            raise JobError("'overlays' is not a dictionary")
+        for overlay, params in self.params["overlays"].items():
+            if overlay == "lava":
+                continue
+            if params.get("format") not in self.OVERLAY_FORMATS:
+                raise JobError(
+                    "Invalid 'format' (%r) for 'overlays.%s'"
+                    % (params.get("format", ""), overlay)
+                )
+            path = params.get("path")
+            if path is None:
+                raise JobError("Missing 'path' for 'overlays.%s'" % overlay)
+            if not path.startswith("/") or ".." in path:
+                raise JobError("Invalid 'path': %r" % path)
+
+        # Check the image format
+        if self.params.get("format") not in self.IMAGE_FORMATS:
+            raise JobError("Unsupported image format %r" % self.params.get("format"))
+
+    def run(self, connection, max_end_time):
+        connection = super().run(connection, max_end_time)
+        if self.params["format"] == "cpio.newc":
+            self.update_cpio()
+        elif self.params["format"] == "ext4":
+            try:
+                self.update_guestfs()
+            except RuntimeError as exc:
+                self.logger.exception(str(exc))
+                raise JobError("Unable to update image %s: %r" % (self.key, str(exc)))
+        else:
+            raise LAVABug("Unknown format %r" % self.params["format"])
+        return connection
+
+    def update_cpio(self):
+        image = self.get_namespace_data(
+            action="download-action", label=self.key, key="file"
+        )
+        compression = self.get_namespace_data(
+            action="download-action", label=self.key, key="compression"
+        )
+        decompressed = self.get_namespace_data(
+            action="download-action", label=self.key, key="decompressed"
+        )
+        self.logger.info("Modifying %r", image)
+        tempdir = self.mkdtemp()
+        # Some images are kept compressed. We should decompress first
+        if compression and not decompressed:
+            self.logger.debug("* decompressing (%s)", compression)
+            image = decompress_file(image, compression)
+        # extract the archive
+        self.logger.debug("* extracting %r", image)
+        uncpio(image, tempdir)
+        os.unlink(image)
+
+        # Add overlays
+        self.logger.debug("Overlays:")
+        for overlay in self.params["overlays"]:
+            label = "%s.%s" % (self.key, overlay)
+            overlay_image = None
+            path = None
+            if overlay == "lava":
+                overlay_image = self.get_namespace_data(
+                    action="compress-overlay", label="output", key="file"
+                )
+                path = "/"
+            else:
+                overlay_image = self.get_namespace_data(
+                    action="download-action", label=label, key="file"
+                )
+                path = self.params["overlays"][overlay]["path"]
+            # Take off initial "/" from path, extract relative to this directory
+            extract_path = os.path.join(tempdir, path[1:])
+            self.logger.debug("- %s: %r to %r", label, overlay_image, extract_path)
+            # In the "validate" function, we check that path startswith '/'
+            # and does not contains '..'
+            untar_file(overlay_image, extract_path)
+
+        # Recreating the archive
+        self.logger.debug("* archiving %r", image)
+        cpio(tempdir, image)
+        if compression and not decompressed:
+            self.logger.debug("* compressing (%s)", compression)
+            image = compress_file(image, compression)
+
+    def update_guestfs(self):
+        image = self.get_namespace_data(
+            action="download-action", label=self.key, key="file"
+        )
+        partition = self.params.get("partition", None)
+        self.logger.info("Modifying %r", image)
+        guest = guestfs.GuestFS(python_return_dict=True)
+        guest.add_drive(image)
+        try:
+            guest.launch()
+            if partition is not None:
+                device = guest.list_partitions()[partition]
+            else:
+                device = guest.list_devices()[0]
+            guest.mount(device, "/")
+        except RuntimeError as exc:
+            self.logger.exception(str(exc))
+            raise JobError("Unable to update image %s: %r" % (self.key, str(exc)))
+
+        self.logger.debug("Overlays:")
+        for overlay in self.params["overlays"]:
+            label = "%s.%s" % (self.key, overlay)
+            overlay_image = None
+            if overlay == "lava":
+                overlay_image = self.get_namespace_data(
+                    action="compress-overlay", label="output", key="file"
+                )
+                path = "/"
+                compress = "gzip"
+            else:
+                overlay_image = self.get_namespace_data(
+                    action="download-action", label=label, key="file"
+                )
+                path = self.params["overlays"][overlay]["path"]
+                compress = None
+            self.logger.debug("- %s: %r to %r", label, overlay_image, path)
+            guest.mkdir_p(path)
+            guest.tar_in(overlay_image, path, compress=compress)
+        guest.umount(device)
+        guest.shutdown()
