@@ -2,6 +2,7 @@
 #
 # Author: Neil Williams <neil.williams@linaro.org>
 #         Remi Duraffort <remi.duraffort@linaro.org>
+#         Antonio Terceiro <antonio.terceiro@linaro.org>
 #
 # This file is part of LAVA Dispatcher.
 #
@@ -26,14 +27,16 @@ import contextlib
 import errno
 import math
 import os
+import pathlib
 import shutil
 import time
 import hashlib
 import requests
 import subprocess  # nosec - verified.
+
 from lava_dispatcher.power import ResetDevice
 from lava_dispatcher.protocols.lxc import LxcProtocol
-from lava_dispatcher.actions.deploy import DeployAction
+from lava_dispatcher.actions.deploy.apply_overlay import AppendOverlays
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.connections.serial import ConnectDevice
 from lava_common.exceptions import InfrastructureError, JobError, LAVABug
@@ -45,6 +48,7 @@ from lava_dispatcher.utils.filesystem import (
     lava_lxc_home,
     copy_overlay_to_lxc,
 )
+from lava_dispatcher.utils.network import requests_retry
 from lava_common.constants import (
     FILE_DOWNLOAD_CHUNK_SIZE,
     HTTP_DOWNLOAD_CHUNK_SIZE,
@@ -53,9 +57,7 @@ from lava_common.constants import (
 from lava_dispatcher.actions.boot.fastboot import EnterFastbootAction
 from lava_dispatcher.actions.boot.u_boot import UBootEnterFastbootAction
 
-from urllib.parse import urlparse
-
-# pylint: disable=logging-not-lazy
+from urllib.parse import quote_plus, urlparse
 
 
 class DownloaderAction(RetryAction):
@@ -68,23 +70,19 @@ class DownloaderAction(RetryAction):
     description = "download with retry"
     summary = "download-retry"
 
-    def __init__(self, key, path, uniquify=True):
+    def __init__(self, key, path, params, uniquify=True):
         super().__init__()
         self.max_retries = 3
         self.key = key  # the key in the parameters of what to download
         self.path = path  # where to download
         self.uniquify = uniquify
+        self.params = params
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
 
         # Find the right action according to the url
-        if "images" in parameters and self.key in parameters["images"]:
-            url = parameters["images"][self.key].get("url")
-        else:
-            url = parameters[self.key].get("url")
+        url = self.params.get("url")
         if url is None:
             raise JobError(
                 "Invalid deploy action: 'url' is missing for '%s'" % self.key
@@ -92,19 +90,39 @@ class DownloaderAction(RetryAction):
 
         url = urlparse(url)
         if url.scheme == "scp":
-            action = ScpDownloadAction(self.key, self.path, url, self.uniquify)
+            action = ScpDownloadAction(
+                self.key, self.path, url, self.uniquify, params=self.params
+            )
         elif url.scheme == "http" or url.scheme == "https":
-            action = HttpDownloadAction(self.key, self.path, url, self.uniquify)
+            action = HttpDownloadAction(
+                self.key, self.path, url, self.uniquify, params=self.params
+            )
         elif url.scheme == "file":
-            action = FileDownloadAction(self.key, self.path, url, self.uniquify)
+            action = FileDownloadAction(
+                self.key, self.path, url, self.uniquify, params=self.params
+            )
         elif url.scheme == "lxc":
             action = LxcDownloadAction(self.key, self.path, url)
+        elif url.scheme == "downloads":
+            action = PreDownloadedAction(self.key, url, self.path, params=self.params)
         else:
             raise JobError("Unsupported url protocol scheme: %s" % url.scheme)
-        self.internal_pipeline.add_action(action)
+        self.pipeline.add_action(action)
+        overlays = self.params.get("overlays", [])
+        for overlay in overlays:
+            if overlay == "lava":
+                # Special case, don't download an overlay, just defined where to find overlays
+                continue
+            self.pipeline.add_action(
+                DownloaderAction(
+                    "%s.%s" % (self.key, overlay), self.path, params=overlays[overlay]
+                )
+            )
+        if overlays:
+            self.pipeline.add_action(AppendOverlays(self.key, params=self.params))
 
 
-class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
+class DownloadHandler(Action):
     """
     The identification of which downloader and whether to
     decompress needs to be done in the validation stage,
@@ -120,17 +138,28 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
     summary = "download-action"
     timeout_exception = InfrastructureError
 
-    def __init__(self, key, path, url, uniquify=True):
+    # Supported decompression commands
+    decompress_command_map = {
+        "bz2": "bunzip2",
+        "gz": "gunzip",
+        "xz": "unxz",
+        "zstd": "unzstd",
+    }
+
+    def __init__(self, key, path, url, uniquify=True, params=None):
         super().__init__()
         self.url = url
         self.key = key
-        # If uniquify is True, store the files in a sub-directory to keep the
-        # path unique.
-        self.path = os.path.join(path, key) if uniquify else path
         self.size = -1
-        self.decompress_command_map = {"xz": "unxz", "gz": "gunzip", "bz2": "bunzip2"}
 
-    def reader(self):  # pylint: disable=no-self-use
+        self.path = path
+        # Store the files in a sub-directory to keep the path unique.
+        if uniquify:
+            self.path = os.path.join(path, key)
+        self.fname = None
+        self.params = params
+
+    def reader(self):
         raise LAVABug("'reader' function unimplemented")
 
     def cleanup(self, connection):
@@ -142,69 +171,66 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
         )
         super().cleanup(connection)
 
-    def _url_to_fname_suffix(self, path, modify):
+    def _compression(self):
+        if self.key == "ramdisk":
+            return False
+        return self.params.get("compression", False)
+
+    def _url_to_fname(self):
+        compression = self._compression()
         filename = os.path.basename(self.url.path)
+
+        # Don't rename files we don't decompress during download
+        if not compression or (compression not in self.decompress_command_map):
+            return os.path.join(self.path, filename)
+
         parts = filename.split(".")
-        # Handle unmodified filename
-        # Also files without suffixes, e.g. kernel images
-        # Don't rename files we don't support decompressing during download
-        if not modify or len(parts) == 1 or (modify not in self.decompress_command_map):
-            return (os.path.join(path, filename), None)
-        return (os.path.join(path, ".".join(parts[:-1])), parts[-1])
+        # Files without suffixes, e.g. kernel images
+        if len(parts) == 1:
+            return os.path.join(self.path, filename)
+        return os.path.join(self.path, ".".join(parts[:-1]))
 
     def validate(self):
         super().validate()
-        if "images" in self.parameters and self.key in self.parameters["images"]:
-            image = self.parameters["images"][self.key]
-            self.url = urlparse(image["url"])
-            compression = image.get("compression")
-            archive = image.get("archive")
-            image_name, _ = self._url_to_fname_suffix(self.path, compression)
-            image_arg = image.get("image_arg")
-            overlay = image.get("overlay", False)
-            self.set_namespace_data(
-                action="download-action", label=self.key, key="file", value=image_name
-            )
+        # Check that self.key is not a path
+        if len(pathlib.Path(self.key).parts) != 1:
+            raise JobError("Invalid key %r" % self.key)
+
+        self.url = urlparse(self.params["url"])
+        compression = self.params.get("compression")
+        archive = self.params.get("archive")
+        overlay = self.params.get("overlay", False)
+        image_arg = self.params.get("image_arg")
+
+        self.fname = self._url_to_fname()
+        if self.fname.endswith("/"):
+            raise JobError("Cannot download a directory for %s" % self.key)
+        # Save into the namespaced data
+        self.set_namespace_data(
+            action="download-action", label=self.key, key="file", value=self.fname
+        )
+        self.set_namespace_data(
+            action="download-action",
+            label=self.key,
+            key="compression",
+            value=compression,
+        )
+        if image_arg is not None:
             self.set_namespace_data(
                 action="download-action",
                 label=self.key,
                 key="image_arg",
                 value=image_arg,
             )
-            self.set_namespace_data(
-                action="download-action",
-                label=self.key,
-                key="compression",
-                value=compression,
-            )
-        else:
-            self.url = urlparse(self.parameters[self.key]["url"])
-            compression = self.parameters[self.key].get("compression")
-            archive = self.parameters[self.key].get("archive")
-            overlay = self.parameters.get("overlay", False)
-            fname, _ = self._url_to_fname_suffix(self.path, compression)
-            if fname.endswith("/"):
-                self.errors = "Cannot download a directory for %s" % self.key
-            self.set_namespace_data(
-                action="download-action", label=self.key, key="file", value=fname
-            )
-            self.set_namespace_data(
-                action="download-action",
-                label=self.key,
-                key="compression",
-                value=compression,
-            )
-
         if overlay:
             self.set_namespace_data(
                 action="download-action", label=self.key, key="overlay", value=overlay
             )
-        if compression:
-            if compression not in ["gz", "bz2", "xz", "zip"]:
-                self.errors = "Unknown 'compression' format '%s'" % compression
-        if archive:
-            if archive not in ["tar"]:
-                self.errors = "Unknown 'archive' format '%s'" % archive
+
+        if compression and compression not in ["gz", "bz2", "xz", "zip", "zstd"]:
+            self.errors = "Unknown 'compression' format '%s'" % compression
+        if archive and archive not in ["tar"]:
+            self.errors = "Unknown 'archive' format '%s'" % archive
         # pass kernel type to boot Action
         if self.key == "kernel" and ("kernel" in self.parameters):
             self.set_namespace_data(
@@ -214,9 +240,23 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                 value=self.parameters[self.key].get("type"),
             )
 
-    def run(
-        self, connection, max_end_time
-    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
+    def _check_checksum(self, algorithm, actual, expected):
+        if expected is None:
+            return
+        if actual == expected:
+            self.results = {"success": {algorithm: actual}}
+            return
+
+        self.logger.error(
+            "%s sum for '%s' does not match", algorithm, self.url.geturl()
+        )
+        self.logger.info("actual  : %s", actual)
+        self.logger.info("expected: %s", expected)
+
+        self.results = {"fail": {algorithm: expected, "download": actual}}
+        raise JobError("%s for '%s' does not match." % (algorithm, self.url.geturl()))
+
+    def run(self, connection, max_end_time):
         def progress_unknown_total(downloaded_sz, last_val):
             """ Compute progress when the size is unknown """
             condition = downloaded_sz >= last_val + 25 * 1024 * 1024
@@ -244,6 +284,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
         # self.cookies = self.job.context.config.lava_cookies  # FIXME: work out how to restore
         md5 = hashlib.md5()  # nosec - not being used for cryptography.
         sha256 = hashlib.sha256()
+        sha512 = hashlib.sha512()
 
         # Create a fresh directory if the old one has been removed by a previous cleanup
         # (when retrying inside a RetryAction)
@@ -255,31 +296,28 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                     "Unable to create %s: %s" % (self.path, str(exc))
                 )
 
-        if "images" in self.parameters and self.key in self.parameters["images"]:
-            remote = self.parameters["images"][self.key]
-            compression = self.parameters["images"][self.key].get("compression", False)
+        compression = self._compression()
+        if self.key == "ramdisk":
+            self.logger.debug("Not decompressing ramdisk as can be used compressed.")
 
-        else:
-            remote = self.parameters[self.key]
-            if self.key == "ramdisk":
-                compression = False
-                self.logger.debug(
-                    "Not decompressing ramdisk as can be used compressed."
-                )
-            else:
-                compression = self.parameters[self.key].get("compression", False)
+        self.set_namespace_data(
+            action="download-action",
+            label=self.key,
+            key="decompressed",
+            value=bool(compression),
+        )
 
-        md5sum = remote.get("md5sum")
-        sha256sum = remote.get("sha256sum")
+        md5sum = self.params.get("md5sum")
+        sha256sum = self.params.get("sha256sum")
+        sha512sum = self.params.get("sha512sum")
 
-        fname, _ = self._url_to_fname_suffix(self.path, compression)
-        if os.path.isdir(fname):
-            raise JobError("Download '%s' is a directory, not a file" % fname)
-        if os.path.exists(fname):
-            os.remove(fname)
+        if os.path.isdir(self.fname):
+            raise JobError("Download '%s' is a directory, not a file" % self.fname)
+        if os.path.exists(self.fname):
+            os.remove(self.fname)
 
-        self.logger.info("downloading %s", remote["url"])
-        self.logger.debug("saving as %s", fname)
+        self.logger.info("downloading %s", self.params["url"])
+        self.logger.debug("saving as %s", self.fname)
 
         downloaded_size = 0
         beginning = time.time()
@@ -290,7 +328,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             progress = progress_unknown_total
         else:
             self.logger.debug(
-                "total size: %d (%dMB)" % (self.size, int(self.size / (1024 * 1024)))
+                "total size: %d (%dMB)", self.size, int(self.size / (1024 * 1024))
             )
             last_value = -5
             progress = progress_known_total
@@ -307,11 +345,11 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                     "Compression %s specified but not decompressing during download",
                     compression,
                 )
-        else:
+        elif not self.params.get("compression", False):
             self.logger.debug("No compression specified")
 
         def update_progress():
-            nonlocal downloaded_size, last_value, md5, sha256
+            nonlocal downloaded_size, last_value, md5, sha256, sha512
             downloaded_size += len(buff)
             (printing, new_value, msg) = progress(downloaded_size, last_value)
             if printing:
@@ -319,15 +357,16 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                 self.logger.debug(msg)
             md5.update(buff)
             sha256.update(buff)
+            sha512.update(buff)
 
         if compression and decompress_command:
             try:
-                with open(fname, "wb") as dwnld_file:
+                with open(self.fname, "wb") as dwnld_file:
                     proc = subprocess.Popen(  # nosec - internal.
                         [decompress_command], stdin=subprocess.PIPE, stdout=dwnld_file
                     )
             except OSError as exc:
-                msg = "Unable to open %s: %s" % (fname, exc.strerror)
+                msg = "Unable to open %s: %s" % (self.fname, exc.strerror)
                 self.logger.error(msg)
                 raise InfrastructureError(msg)
 
@@ -347,7 +386,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                         raise JobError(error_message)
             proc.wait()
         else:
-            with open(fname, "wb") as dwnld_file:
+            with open(self.fname, "wb") as dwnld_file:
                 for buff in self.reader():
                     update_progress()
                     dwnld_file.write(buff)
@@ -355,27 +394,27 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
         # Log the download speed
         ending = time.time()
         self.logger.info(
-            "%dMB downloaded in %0.2fs (%0.2fMB/s)"
-            % (
-                downloaded_size / (1024 * 1024),
-                round(ending - beginning, 2),
-                round(downloaded_size / (1024 * 1024 * (ending - beginning)), 2),
-            )
+            "%dMB downloaded in %0.2fs (%0.2fMB/s)",
+            downloaded_size / (1024 * 1024),
+            round(ending - beginning, 2),
+            round(downloaded_size / (1024 * 1024 * (ending - beginning)), 2),
         )
 
         # If the remote server uses "Content-Encoding: gzip", this calculation will be wrong
         # because requests will decompress the file on the fly, creating a larger file than
         # LAVA expects.
-        if self.size > 0:
-            if self.size != downloaded_size:
-                raise InfrastructureError(
-                    "Download finished (%i bytes) but was not expected size (%i bytes), check your networking."
-                    % (downloaded_size, self.size)
-                )
+        if self.size > 0 and self.size != downloaded_size:
+            raise InfrastructureError(
+                "Download finished (%i bytes) but was not expected size (%i bytes), check your networking."
+                % (downloaded_size, self.size)
+            )
 
         # set the dynamic data into the context
         self.set_namespace_data(
-            action="download-action", label=self.key, key="file", value=fname
+            action="download-action", label=self.key, key="file", value=self.fname
+        )
+        self.set_namespace_data(
+            action="download-action", label="file", key=self.key, value=self.fname
         )
         self.set_namespace_data(
             action="download-action", label=self.key, key="md5", value=md5.hexdigest()
@@ -386,72 +425,38 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             key="sha256",
             value=sha256.hexdigest(),
         )
+        self.set_namespace_data(
+            action="download-action",
+            label=self.key,
+            key="sha512",
+            value=sha512.hexdigest(),
+        )
 
         # handle archive files
-        if "images" in self.parameters and self.key in self.parameters["images"]:
-            archive = self.parameters["images"][self.key].get("archive", False)
-        else:
-            archive = self.parameters[self.key].get("archive")
+        archive = self.params.get("archive")
         if archive:
-            origin = fname
-            target_fname = os.path.basename(origin).rstrip("." + archive)
-            target_fname_path = os.path.join(os.path.dirname(origin), target_fname)
-            if os.path.exists(target_fname_path):
-                os.remove(target_fname_path)
+            if archive != "tar":
+                raise JobError("Unknown archive format %r" % archive)
 
-            if archive == "tar":
-                untar_file(origin, None, member=target_fname, outfile=target_fname_path)
-                self.set_namespace_data(
-                    action="download-action",
-                    label=self.key,
-                    key="file",
-                    value=target_fname_path,
-                )
-                self.set_namespace_data(
-                    action="download-action",
-                    label="file",
-                    key=self.key,
-                    value=target_fname,
-                )
-            self.logger.debug("Using %s archive" % archive)
-
-        if md5sum is not None:
-            chk_md5sum = self.get_namespace_data(
-                action="download-action", label=self.key, key="md5"
+            target_fname_path = os.path.join(os.path.dirname(self.fname), self.key)
+            self.logger.debug("Extracting %s archive in %s", archive, target_fname_path)
+            untar_file(self.fname, target_fname_path)
+            self.set_namespace_data(
+                action="download-action",
+                label=self.key,
+                key="file",
+                value=target_fname_path,
             )
-            if md5sum != chk_md5sum:
-                self.logger.error("md5sum of downloaded content: %s" % chk_md5sum)
-                self.logger.info(
-                    "sha256sum of downloaded content: %s"
-                    % (
-                        self.get_namespace_data(
-                            action="download-action", label=self.key, key="sha256"
-                        )
-                    )
-                )
-                self.results = {"fail": {"md5": md5sum, "download": chk_md5sum}}
-                raise JobError("MD5 checksum for '%s' does not match." % fname)
-            self.results = {"success": {"md5": md5sum}}
-
-        if sha256sum is not None:
-            chk_sha256sum = self.get_namespace_data(
-                action="download-action", label=self.key, key="sha256"
+            self.set_namespace_data(
+                action="download-action",
+                label="file",
+                key=self.key,
+                value=target_fname_path,
             )
-            if sha256sum != chk_sha256sum:
-                self.logger.info(
-                    "md5sum of downloaded content: %s"
-                    % (
-                        self.get_namespace_data(
-                            action="download-action", label=self.key, key="md5"
-                        )
-                    )
-                )
-                self.logger.error("sha256sum of downloaded content: %s" % chk_sha256sum)
-                self.results = {
-                    "fail": {"sha256": sha256sum, "download": chk_sha256sum}
-                }
-                raise JobError("SHA256 checksum for '%s' does not match." % fname)
-            self.results = {"success": {"sha256": sha256sum}}
+
+        self._check_checksum("md5", md5.hexdigest(), md5sum)
+        self._check_checksum("sha256", sha256.hexdigest(), sha256sum)
+        self._check_checksum("sha512", sha512.hexdigest(), sha512sum)
 
         # certain deployments need prefixes set
         if self.parameters["to"] == "tftp" or self.parameters["to"] == "nbd":
@@ -462,7 +467,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                 action="download-action",
                 label="file",
                 key=self.key,
-                value=os.path.join(suffix, self.key, os.path.basename(fname)),
+                value=os.path.join(suffix, self.key, os.path.basename(self.fname)),
             )
         elif self.parameters["to"] == "iso-installer":
             suffix = self.get_namespace_data(
@@ -472,11 +477,7 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                 action="download-action",
                 label="file",
                 key=self.key,
-                value=os.path.join(suffix, self.key, os.path.basename(fname)),
-            )
-        else:
-            self.set_namespace_data(
-                action="download-action", label="file", key=self.key, value=fname
+                value=os.path.join(suffix, self.key, os.path.basename(self.fname)),
             )
 
         # xnbd protocoll needs to know the location
@@ -499,6 +500,11 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                     action="download-action", label=self.key, key="sha256"
                 )
             ),
+            "sha512sum": str(
+                self.get_namespace_data(
+                    action="download-action", label=self.key, key="sha512"
+                )
+            ),
         }
         return connection
 
@@ -523,20 +529,16 @@ class FileDownloadAction(DownloadHandler):
             )
 
     def reader(self):
-        reader = None
         try:
-            reader = open(self.url.path, "rb")
-            buff = reader.read(FILE_DOWNLOAD_CHUNK_SIZE)
-            while buff:
-                yield buff
+            with open(self.url.path, "rb") as reader:
                 buff = reader.read(FILE_DOWNLOAD_CHUNK_SIZE)
+                while buff:
+                    yield buff
+                    buff = reader.read(FILE_DOWNLOAD_CHUNK_SIZE)
         except OSError as exc:
             raise InfrastructureError(
-                "Unable to write to %s: %s" % (self.url.path, str(exc))
+                "Unable to read from %s: %s" % (self.url.path, str(exc))
             )
-        finally:
-            if reader is not None:
-                reader.close()
 
 
 class HttpDownloadAction(DownloadHandler):
@@ -552,27 +554,40 @@ class HttpDownloadAction(DownloadHandler):
         super().validate()
         res = None
         try:
+            http_cache = self.job.parameters["dispatcher"].get(
+                "http_url_format_string", ""
+            )
+            if http_cache:
+                self.logger.info("Using caching service: '%s'", http_cache)
+                try:
+                    self.url = urlparse(http_cache % quote_plus(self.url.geturl()))
+                except TypeError as exc:
+                    self.logger.error("Invalid http_url_format_string: '%s'", exc)
+                    self.errors = "Invalid http_url_format_string: '%s'" % str(exc)
+                    return
+
             self.logger.debug("Validating that %s exists", self.url.geturl())
             # Force the non-use of Accept-Encoding: gzip, this will permit to know the final size
-            res = requests.head(
+            res = requests_retry().head(
                 self.url.geturl(), allow_redirects=True, headers={"Accept-Encoding": ""}
             )
-            if res.status_code != requests.codes.OK:  # pylint: disable=no-member
+            if res.status_code != requests.codes.OK:
                 # try using (the slower) get for services with broken redirect support
                 self.logger.debug("Using GET because HEAD is not supported properly")
                 res.close()
                 # Like for HEAD, we need get a size, so disable gzip
-                res = requests.get(
+                res = requests_retry().get(
                     self.url.geturl(),
                     allow_redirects=True,
                     stream=True,
                     headers={"Accept-Encoding": ""},
                 )
-                if res.status_code != requests.codes.OK:  # pylint: disable=no-member
+                if res.status_code != requests.codes.OK:
                     self.errors = "Resource unavailable at '%s' (%d)" % (
                         self.url.geturl(),
                         res.status_code,
                     )
+                    return
 
             self.size = int(res.headers.get("content-length", -1))
         except requests.Timeout:
@@ -590,8 +605,10 @@ class HttpDownloadAction(DownloadHandler):
         try:
             # FIXME: When requests 3.0 is released, use the enforce_content_length
             # parameter to raise an exception the file is not fully downloaded
-            res = requests.get(self.url.geturl(), allow_redirects=True, stream=True)
-            if res.status_code != requests.codes.OK:  # pylint: disable=no-member
+            res = requests_retry().get(
+                self.url.geturl(), allow_redirects=True, stream=True
+            )
+            if res.status_code != requests.codes.OK:
                 # This is an Infrastructure error because the validate function
                 # checked that the file does exist.
                 raise InfrastructureError(
@@ -621,7 +638,7 @@ class ScpDownloadAction(DownloadHandler):
         super().validate()
         try:
             size = subprocess.check_output(  # nosec - internal.
-                ["nice", "ssh", self.url.netloc, "stat", "-c", "%s", self.url.path],
+                ["ssh", self.url.netloc, "stat", "-c", "%s", self.url.path],
                 stderr=subprocess.STDOUT,
             )
             self.size = int(size)
@@ -632,8 +649,7 @@ class ScpDownloadAction(DownloadHandler):
         process = None
         try:
             process = subprocess.Popen(  # nosec - internal.
-                ["nice", "ssh", self.url.netloc, "cat", self.url.path],
-                stdout=subprocess.PIPE,
+                ["ssh", self.url.netloc, "cat", self.url.path], stdout=subprocess.PIPE
             )
             buff = process.stdout.read(SCP_DOWNLOAD_CHUNK_SIZE)
             while buff:
@@ -692,7 +708,7 @@ class LxcDownloadAction(Action):
         fname = os.path.basename(self.url.path)
         lxc_home = lava_lxc_home(lxc_name, self.job.parameters["dispatcher"])
         file_path = os.path.join(lxc_home, fname)
-        self.logger.debug("Found '%s' matching '%s'", file_path, fname)
+        self.logger.debug("Trying '%s' matching '%s'", file_path, fname)
         if os.path.exists(file_path):
             self.set_namespace_data(
                 action="download-action", label=self.key, key="file", value=file_path
@@ -702,6 +718,72 @@ class LxcDownloadAction(Action):
         return connection
 
 
+class PreDownloadedAction(Action):
+    """
+    Maps references to files in downloads:// to their full path.
+    """
+
+    name = "pre-downloaded"
+    description = "Map to the correct downloaded path"
+    summary = "pre downloaded"
+
+    def __init__(self, key, url, path, params=None):
+        super().__init__()
+        self.key = key
+        self.url = url
+        self.path = path
+        self.params = params
+
+    def validate(self):
+        super().validate()
+        if self.url.scheme != "downloads":
+            self.errors = "downloads:// url scheme is invalid"
+        if not self.url.path and not self.url.hostname:
+            self.errors = "Invalid path in downloads:// url"
+
+        image_arg = self.params.get("image_arg")
+        if image_arg is not None:
+            self.set_namespace_data(
+                action="download-action",
+                label=self.key,
+                key="image_arg",
+                value=image_arg,
+            )
+
+    def run(self, connection, max_end_time):
+        # In downloads://foo/bar.ext, "foo" is the "hostname", "/bar.ext" is
+        # the path. But in downloads://foo.ext, "foo.ext" is the hostnane, and
+        # the path is empty.
+        filename = self.url.hostname
+        if self.url.path:
+            filename += self.url.path
+
+        namespace = self.parameters["namespace"]
+        top = pathlib.Path(self.job.tmp_dir) / "downloads" / namespace
+        src = top / filename
+        if not src.exists():
+            existing = [
+                str(f).replace(str(top) + "/", "")
+                for f in top.rglob("*")
+                if not f.is_dir()
+            ]
+            if existing:
+                available = "available: " + ", ".join(existing)
+            else:
+                available = "not files at all available"
+            raise JobError(f"Resource unavailable: {filename} ({available})")
+
+        dest = pathlib.Path(self.path) / filename
+        dest.parent.mkdir(parents=True, exist_ok=True)
+        os.link(src, dest)
+
+        self.set_namespace_data(
+            action="download-action", label=self.key, key="file", value=str(dest)
+        )
+
+        return connection
+
+
 class QCowConversionAction(Action):
     """
     explicit action for qcow conversion to avoid reliance
@@ -756,12 +838,9 @@ class Download(Deployment):
     compatibility = 1
     name = "download"
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = DownloadAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return DownloadAction()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -772,7 +851,7 @@ class Download(Deployment):
         return True, "accepted"
 
 
-class DownloadAction(DeployAction):  # pylint:disable=too-many-instance-attributes
+class DownloadAction(Action):
 
     name = "download-deploy"
     description = "download files and copy to LXC if available"
@@ -789,9 +868,7 @@ class DownloadAction(DeployAction):  # pylint:disable=too-many-instance-attribut
         )
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         # Check if the device has a power command such as HiKey, Dragonboard,
         # etc. against device that doesn't like Nexus, etc.
         # This is required in order to power on the device so that when the
@@ -801,27 +878,28 @@ class DownloadAction(DeployAction):  # pylint:disable=too-many-instance-attribut
         #
         # NOTE: Add more power on strategies, if required for specific devices.
         if self.job.device.get("fastboot_via_uboot", False):
-            self.internal_pipeline.add_action(ConnectDevice())
-            self.internal_pipeline.add_action(UBootEnterFastbootAction())
+            self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(UBootEnterFastbootAction())
         elif self.job.device.hard_reset_command:
             self.force_prompt = True
-            self.internal_pipeline.add_action(ConnectDevice())
-            self.internal_pipeline.add_action(ResetDevice())
+            self.pipeline.add_action(ConnectDevice())
+            self.pipeline.add_action(ResetDevice())
         else:
-            self.internal_pipeline.add_action(EnterFastbootAction())
+            self.pipeline.add_action(EnterFastbootAction())
 
         self.download_dir = self.mkdtemp()
-        image_keys = sorted(parameters["images"].keys())
-        for image in image_keys:
-            self.internal_pipeline.add_action(
-                DownloaderAction(image, self.download_dir)
+        for image in sorted(parameters["images"].keys()):
+            self.pipeline.add_action(
+                DownloaderAction(
+                    image, self.download_dir, params=parameters["images"][image]
+                )
             )
         if self.test_needs_overlay(parameters):
-            self.internal_pipeline.add_action(OverlayAction())
-        self.internal_pipeline.add_action(CopyToLxcAction())
+            self.pipeline.add_action(OverlayAction())
+        self.pipeline.add_action(CopyToLxcAction())
 
 
-class CopyToLxcAction(DeployAction):
+class CopyToLxcAction(Action):
     """
     Copy downloaded files to LXC within LAVA_LXC_HOME.
     """
@@ -835,17 +913,17 @@ class CopyToLxcAction(DeployAction):
         self.retries = 3
         self.sleep = 10
 
-    def run(self, connection, max_end_time):  # pylint: disable=too-many-locals
+    def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         # this is the device namespace - the lxc namespace is not accessible
         lxc_name = None
-        protocol = [
+        protocols = [
             protocol
             for protocol in self.job.protocols
             if protocol.name == LxcProtocol.name
-        ][0]
-        if protocol:
-            lxc_name = protocol.lxc_name
+        ]
+        if protocols:
+            lxc_name = protocols[0].lxc_name
         else:
             return connection
 
