@@ -23,7 +23,6 @@ import stat
 import glob
 import shutil
 import tarfile
-from lava_dispatcher.actions.deploy import DeployAction
 from lava_dispatcher.action import Action, Pipeline
 from lava_common.exceptions import InfrastructureError, LAVABug
 from lava_dispatcher.actions.deploy.testdef import TestDefinitionAction
@@ -40,12 +39,9 @@ class Overlay(Deployment):
     compatibility = 4
     name = "overlay"
 
-    def __init__(self, parent, parameters):
-        super().__init__(parent)
-        self.action = OverlayAction()
-        self.action.section = self.action_type
-        self.action.job = self.job
-        parent.add_action(self.action, parameters)
+    @classmethod
+    def action(cls):
+        return OverlayAction()
 
     @classmethod
     def accepts(cls, device, parameters):
@@ -56,8 +52,7 @@ class Overlay(Deployment):
         return True, "accepted"
 
 
-# pylint: disable=too-many-instance-attributes
-class OverlayAction(DeployAction):
+class CreateOverlay(Action):
     """
     Creates a temporary location into which the lava test shell scripts are installed.
     The location remains available for the testdef actions to populate
@@ -78,7 +73,7 @@ class OverlayAction(DeployAction):
     overlays are handled by TestDefinitionAction.
     """
 
-    name = "lava-overlay"
+    name = "lava-create-overlay"
     description = "add lava scripts during deployment for test shell use"
     summary = "overlay the lava support scripts"
 
@@ -102,9 +97,7 @@ class OverlayAction(DeployAction):
         self.scripts_to_copy = sorted(
             glob.glob(os.path.join(self.lava_test_dir, "lava-*"))
         )
-        # Distro-specific scripts override the generic ones
-        if not self.test_needs_overlay(self.parameters):
-            return
+
         lava_test_results_dir = self.get_constant("lava_test_results_dir", "posix")
         lava_test_results_dir = lava_test_results_dir % self.job.job_id
         self.set_namespace_data(
@@ -122,7 +115,7 @@ class OverlayAction(DeployAction):
         )
 
         # Add distro support scripts - only if deployment_data is set
-        distro = self.parameters["deployment_data"].get("distro")
+        distro = self.parameters.get("deployment_data", {}).get("distro")
         if distro:
             distro_support_dir = "%s/distro/%s" % (self.lava_test_dir, distro)
             self.scripts_to_copy += sorted(
@@ -147,21 +140,17 @@ class OverlayAction(DeployAction):
                 break
 
     def populate(self, parameters):
-        self.internal_pipeline = Pipeline(
-            parent=self, job=self.job, parameters=parameters
-        )
-        if self.test_needs_overlay(parameters):
-            if any(
-                "ssh" in data
-                for data in self.job.device["actions"]["deploy"]["methods"]
-            ):
-                # only devices supporting ssh deployments add this action.
-                self.internal_pipeline.add_action(SshAuthorize())
-            self.internal_pipeline.add_action(VlandOverlayAction())
-            self.internal_pipeline.add_action(MultinodeOverlayAction())
-            self.internal_pipeline.add_action(TestDefinitionAction())
-            self.internal_pipeline.add_action(CompressOverlay())
-            self.internal_pipeline.add_action(PersistentNFSOverlay())  # idempotent
+        self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        if any(
+            "ssh" in data for data in self.job.device["actions"]["deploy"]["methods"]
+        ):
+            # only devices supporting ssh deployments add this action.
+            self.pipeline.add_action(SshAuthorize())
+        self.pipeline.add_action(VlandOverlayAction())
+        self.pipeline.add_action(MultinodeOverlayAction())
+        self.pipeline.add_action(TestDefinitionAction())
+        self.pipeline.add_action(CompressOverlay())
+        self.pipeline.add_action(PersistentNFSOverlay())  # idempotent
 
     def _export_data(self, fout, data, prefix):
         if isinstance(data, dict):
@@ -184,17 +173,7 @@ class OverlayAction(DeployAction):
             self.logger.debug("- %s=%s", prefix, data)
             fout.write("export %s=%s\n" % (prefix, data))
 
-    def run(self, connection, max_end_time):  # pylint: disable=too-many-locals
-        """
-        Check if a lava-test-shell has been requested, implement the overlay
-        * create test runner directories beneath the temporary location
-        * copy runners into test runner directories
-        """
-        namespace = self.parameters.get("namespace")
-        if not self.test_needs_overlay(self.parameters):
-            self.logger.info("[%s] skipped %s - no test action.", namespace, self.name)
-            return connection
-
+    def run(self, connection, max_end_time):
         tmp_dir = self.mkdtemp()
         self.set_namespace_data(
             action="test", label="shared", key="location", value=tmp_dir
@@ -207,6 +186,7 @@ class OverlayAction(DeployAction):
         shell = self.get_namespace_data(
             action="test", label="shared", key="lava_test_sh_cmd"
         )
+        namespace = self.parameters.get("namespace")
         self.logger.debug("[%s] Preparing overlay tarball in %s", namespace, tmp_dir)
         lava_path = os.path.abspath("%s/%s" % (tmp_dir, lava_test_results_dir))
         for runner_dir in ["bin", "tests", "results"]:
@@ -260,6 +240,18 @@ class OverlayAction(DeployAction):
                 if data:
                     self.logger.debug("%s:", source)
                     self._export_data(fout, data, prefix)
+            data = None
+            if (
+                "protocols" in self.job.parameters
+                and "lava-multinode" in self.job.parameters["protocols"]
+                and "environment" in self.job.parameters["protocols"]["lava-multinode"]
+            ):
+                data = self.job.parameters["protocols"]["lava-multinode"]["environment"]
+            elif "environment" in self.job.parameters:
+                data = self.job.parameters["environment"]
+            if data:
+                self.logger.debug("job environment:")
+                self._export_data(fout, data, "")
 
         # Generate the file containing the secrets
         if "secrets" in self.job.parameters:
@@ -272,6 +264,44 @@ class OverlayAction(DeployAction):
         return connection
 
 
+class OverlayAction(CreateOverlay):
+    """
+    Creates an overlay, but only if it has been requested by any of the test
+    actions (in contrast with CreateOverlay, that creates the overlay
+    unconditionally).
+    """
+
+    name = "lava-overlay"
+    description = "add lava scripts during deployment for test shell use"
+    summary = "overlay the lava support scripts"
+
+    def validate(self):
+        if not self.test_needs_overlay(self.parameters):
+            return
+
+        super().validate()
+
+    def populate(self, parameters):
+        if not self.test_needs_overlay(parameters):
+            self.pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+            return
+
+        super().populate(parameters)
+
+    def run(self, connection, max_end_time):
+        """
+        Check if a lava-test-shell has been requested, implement the overlay
+        * create test runner directories beneath the temporary location
+        * copy runners into test runner directories
+        """
+        if not self.test_needs_overlay(self.parameters):
+            namespace = self.parameters.get("namespace")
+            self.logger.info("[%s] skipped %s - no test action.", namespace, self.name)
+            return connection
+
+        return super().run(connection, max_end_time)
+
+
 class MultinodeOverlayAction(OverlayAction):
 
     name = "lava-multinode-overlay"
@@ -310,9 +340,7 @@ class MultinodeOverlayAction(OverlayAction):
             else:
                 self.role = self.job.parameters["protocols"][self.protocol]["role"]
 
-    def run(
-        self, connection, max_end_time
-    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
+    def run(self, connection, max_end_time):
         if self.role is None:
             self.logger.debug("skipped %s", self.name)
             return connection
@@ -460,7 +488,6 @@ class VlandOverlayAction(OverlayAction):
             for tag in device_params[interface]["tags"]:
                 self.tags.append(",".join([interface, tag]))
 
-    # pylint: disable=anomalous-backslash-in-string
     def run(self, connection, max_end_time):
         """
         Writes out file contents from lists, across multiple lines
@@ -667,7 +694,6 @@ class PersistentNFSOverlay(Action):
     so that it can be mounted later when the overlay is applied.
     """
 
-    section = "deploy"
     name = "persistent-nfs-overlay"
     description = "unpack overlay into persistent NFS"
     summary = "add test overlay to NFS"
@@ -695,3 +721,6 @@ class PersistentNFSOverlay(Action):
         self.set_namespace_data(
             action=self.name, label="nfs_address", key="serverip", value=nfs_server
         )
+
+        self.job.device["dynamic_data"]["NFS_ROOTFS"] = dirname
+        self.job.device["dynamic_data"]["NFS_SERVER_IP"] = nfs_server
