@@ -27,10 +27,9 @@ import pexpect
 from lava_common.compat import yaml_safe_dump
 from lava_common.decorators import nottest
 from lava_common.exceptions import (
+    ConnectionClosedError,
     JobError,
-    InfrastructureError,
     TestError,
-    LAVABug,
     LAVATimeoutError,
 )
 from lava_dispatcher.action import Action, Pipeline
@@ -229,7 +228,14 @@ class TestShellAction(Action):
         )
 
         if not connection:
-            raise LAVABug("No connection retrieved from namespace data")
+            self.logger.error(
+                "No connection to the DUT, Check that a boot action preced test actions"
+            )
+            if connection_namespace:
+                self.logger.error(
+                    "Also check actions in namespace %r", connection_namespace
+                )
+            raise JobError("No connection to the DUT")
 
         self.signal_director.connection = connection
 
@@ -620,7 +626,7 @@ class TestShellAction(Action):
 
         elif event == "eof":
             self.testset_name = None
-            raise InfrastructureError("lava_test_shell connection dropped.")
+            raise ConnectionClosedError("Connection closed")
 
         elif event == "timeout":
             # allow feedback in long runs
