@@ -61,7 +61,7 @@ class WaitUSBSerialDeviceAction(Action):
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         self.logger.debug("Waiting for usb serial device: %s", self.serial_device)
-        wait_udev_event(action="add", match_dict=self.serial_device, subsystem="tty")
+        wait_udev_event(match_dict=self.serial_device, subsystem="tty")
         if self.usb_sleep:
             self.logger.debug(
                 "Waiting for the board to setup, sleeping %ds", self.usb_sleep
@@ -102,10 +102,7 @@ class WaitDFUDeviceAction(Action):
         connection = super().run(connection, max_end_time)
         self.logger.debug("Waiting for DFU device: %s", self.dfu_device)
         wait_udev_event(
-            action="add",
-            match_dict=self.dfu_device,
-            subsystem="usb",
-            devtype="usb_device",
+            match_dict=self.dfu_device, subsystem="usb", devtype="usb_device"
         )
         return connection
 
@@ -132,10 +129,7 @@ class WaitUSBMassStorageDeviceAction(Action):
         connection = super().run(connection, max_end_time)
         self.logger.debug("Waiting for USB mass storage device: %s", self.ms_device)
         wait_udev_event(
-            action="add",
-            match_dict=self.ms_device,
-            subsystem="block",
-            devtype="partition",
+            match_dict=self.ms_device, subsystem="block", devtype="partition"
         )
         return connection
 
@@ -159,7 +153,7 @@ class WaitDevicePathAction(Action):
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         self.logger.debug("Waiting for udev device path: %s", self.devicepath)
-        wait_udev_event(action="add", devicepath=self.devicepath)
+        wait_udev_event(devicepath=self.devicepath)
         return connection
 
 
@@ -187,25 +181,42 @@ class WaitDeviceBoardID(Action):
     def run(self, connection, max_end_time):
         connection = super().run(connection, max_end_time)
         self.logger.debug("Waiting for udev device with ID: %s", self.board_id)
-        wait_udev_event(action="add", match_dict=self.udev_device)
+        wait_udev_event(match_dict=self.udev_device)
         return connection
 
 
-def _dict_compare(d1, d2):  # pylint: disable=invalid-name
+def _dict_compare(d1, d2):
     d1_keys = set(d1.keys())
     d2_keys = set(d2.keys())
     intersect_keys = d1_keys.intersection(d2_keys)
     return set(o for o in intersect_keys if d1[o] == d2[o])
 
 
-def wait_udev_event(
-    action="add", match_dict=None, subsystem=None, devtype=None, devicepath=None
-):
-    if action not in ["add", "remove", "change"]:
-        raise LAVABug(
-            "Invalid action for udev to wait for: %s, expected 'add' or 'remove'"
-            % action
-        )
+def match(device, match_dict, devicepath):
+    same = _dict_compare(dict(device), match_dict)
+    if same == set(match_dict.keys()):
+        if devicepath:
+            if devicepath in dict(device).get("DEVLINKS", "") or devicepath in dict(
+                device
+            ).get("DEVNAME", ""):
+                return True
+        else:
+            return True
+    return False
+
+
+def wait_udev_event_setup(devicepath, devtype, match_dict, subsystem):
+    """
+    Setup pyudev internals for use by wait_udev_event and wait_udev_change_event methods
+    :param devicepath:
+    :param devtype:
+    :param match_dict:
+    :param subsystem:
+    :return: (context, match_dict, monitor)
+        context is a pyudev.Context instance
+        match_dict from input parameter (initialized to dict if unset)
+        monitor is pyudev.Monitor instance
+    """
     if match_dict:
         if not isinstance(match_dict, dict):
             raise LAVABug("match_dict was not a dict")
@@ -218,7 +229,7 @@ def wait_udev_event(
             raise LAVABug("Neither match_dict nor devicepath were set")
     if devtype and not subsystem:
         raise LAVABug("Cannot filter udev by devtype without a subsystem")
-    match_dict["ACTION"] = action
+    # Create and configure the monitor
     context = pyudev.Context()
     monitor = pyudev.Monitor.from_netlink(context)
     if devtype and subsystem:
@@ -226,16 +237,43 @@ def wait_udev_event(
     else:
         if subsystem:
             monitor.filter_by(subsystem)
+    return context, match_dict, monitor
+
+
+def wait_udev_event(match_dict=None, subsystem=None, devtype=None, devicepath=None):
+    context, match_dict, monitor = wait_udev_event_setup(
+        devicepath, devtype, match_dict, subsystem
+    )
+
+    # Start to listen for events
+    monitor.start()
+
+    # Check if the device is already plugged-in
+    for device in context.list_devices():
+        if match(device, match_dict, devicepath):
+            return
+
+    # Wait for events.
+    # Every events that happened since the call to start() will be handled now.
+    match_dict["ACTION"] = "add"
     for device in iter(monitor.poll, None):
-        same = _dict_compare(dict(device), match_dict)
-        if same == set(match_dict.keys()):
-            if devicepath:
-                if devicepath in dict(device).get("DEVLINKS", "") or devicepath in dict(
-                    device
-                ).get("DEVNAME", ""):
-                    break
-            else:
-                break
+        if match(device, match_dict, devicepath):
+            return
+
+
+def wait_udev_changed_event(
+    match_dict=None, subsystem=None, devtype=None, devicepath=None
+):
+    context, match_dict, monitor = wait_udev_event_setup(
+        devicepath, devtype, match_dict, subsystem
+    )
+
+    # Start to listen for events
+    monitor.start()
+    match_dict["ACTION"] = "change"
+    for device in iter(monitor.poll, None):
+        if match(device, match_dict, devicepath):
+            return
 
 
 def get_udev_devices(job=None, logger=None, device_info=None):
@@ -340,69 +378,3 @@ def allow_fs_label(device):
         ):
             _allow_fs_label = True
     return _allow_fs_label
-
-
-def lxc_udev_rule(data):
-    """Construct the udev rule string."""
-    rule = 'ACTION=="add", '
-    if data["serial_number"]:
-        rule += 'ATTR{{serial}}=="{serial_number}", '
-    if data["vendor_id"] is not None:
-        rule += 'ATTR{{idVendor}}=="{vendor_id}", '
-    if data["product_id"] is not None:
-        rule += 'ATTR{{idProduct}}=="{product_id}", '
-    if data["fs_label"] is not None:
-        rule += 'ENV{{ID_FS_LABEL}}=="{fs_label}", '
-    rule += (
-        'RUN+="/usr/share/lava-dispatcher/lava_lxc_device_add.py '
-        "--lxc-name {lxc_name} --device-node $name "
-        "--job-id {job_id}"
-    )
-    if data["logging_url"]:
-        rule += " --logging-url {logging_url}"
-    rule = rule.format(**data)
-
-    if data["master_cert"] is not None:
-        rule += " --master-cert %s" % data["master_cert"]
-    if data["slave_cert"] is not None:
-        rule += " --slave-cert %s" % data["slave_cert"]
-    if data["socks_proxy"] is not None:
-        rule += " --socks-proxy %s" % data["socks_proxy"]
-    if data["ipv6"]:
-        rule += " --ipv6"
-    if data["fs_label"] is not None:
-        rule = 'IMPORT{builtin}="blkid"\n' + rule
-    rule += '"\n'
-    return rule
-
-
-def lxc_udev_rule_parent(data):
-    """Construct the udev rule string."""
-    rule = 'ACTION=="add", '
-    if data["vendor_id"] is not None:
-        rule += 'ATTRS{{idVendor}}=="{vendor_id}", '
-    if data["product_id"] is not None:
-        rule += 'ATTRS{{idProduct}}=="{product_id}", '
-    if data["fs_label"] is not None:
-        rule += 'ENV{{ID_FS_LABEL}}=="{fs_label}", '
-    rule += (
-        'RUN+="/usr/share/lava-dispatcher/lava_lxc_device_add.py '
-        "--lxc-name {lxc_name} --device-node $name "
-        "--job-id {job_id}"
-    )
-    if data["logging_url"]:
-        rule += " --logging-url {logging_url}"
-    rule = rule.format(**data)
-
-    if data["master_cert"] is not None:
-        rule += " --master-cert %s" % data["master_cert"]
-    if data["slave_cert"] is not None:
-        rule += " --slave-cert %s" % data["slave_cert"]
-    if data["socks_proxy"] is not None:
-        rule += " --socks-proxy %s" % data["socks_proxy"]
-    if data["ipv6"]:
-        rule += " --ipv6"
-    if data["fs_label"] is not None:
-        rule = 'IMPORT{builtin}="blkid"\n' + rule
-    rule += '"\n'
-    return rule
