@@ -17,8 +17,6 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
-# pylint: disable=no-member,too-many-locals,too-many-nested-blocks,
-# pylint: disable=too-many-return-statements,ungrouped-imports
 
 import hashlib
 import os
@@ -30,7 +28,7 @@ from urllib.parse import quote
 from collections import OrderedDict  # pylint: disable=unused-import
 
 from lava_common.compat import yaml_load, yaml_safe_load
-from lava_common.utils import debian_package_version
+from lava_common.version import __version__
 from lava_results_app.models import (
     TestSuite,
     TestSet,
@@ -39,7 +37,6 @@ from lava_results_app.models import (
     ActionData,
     MetaType,
 )
-from django.core.exceptions import MultipleObjectsReturned
 from lava_common.timeout import Timeout
 
 
@@ -111,9 +108,7 @@ def create_metadata_store(results, job):
     return meta_filename
 
 
-def map_scanned_results(
-    results, job, markers, meta_filename
-):  # pylint: disable=too-many-branches,too-many-statements,too-many-return-statements
+def map_scanned_results(results, job, markers, meta_filename):
     """
     Sanity checker on the logged results dictionary
     :param results: results logged via the slave
@@ -140,7 +135,7 @@ def map_scanned_results(
     metadata = yaml.dump(results)
     if len(metadata) > 4096:  # bug 2471 - test_length unit test
         msg = "[%d] Result metadata is too long. %s" % (job.id, metadata)
-        logger.error(msg)
+        logger.warning(msg)
         append_failure_comment(job, msg)
         metadata = ""
 
@@ -236,15 +231,11 @@ def _get_job_metadata(job):
         }
     )
     # Add lava-server-version to metadata
-    packaged = debian_package_version(pkg="lava-server")
-    if packaged:
-        retval.update({"lava-server-version": packaged})
+    retval.update({"lava-server-version": __version__})
     return retval
 
 
-def _get_action_metadata(
-    data
-):  # pylint: disable=too-many-branches,too-many-nested-blocks,too-many-statements
+def _get_action_metadata(data):
     if not isinstance(data, list):
         return None
     retval = {}
@@ -424,12 +415,7 @@ def map_metadata(description, job):
     except yaml.YAMLError as exc:
         logger.exception("[%s] %s", job.id, exc)
         return False
-    try:
-        testdata, created = TestData.objects.get_or_create(testjob=job)
-    except MultipleObjectsReturned:
-        # only happens for small number of jobs affected by original bug.
-        logger.info("[%s] skipping alteration of duplicated TestData", job.id)
-        return False
+    testdata, created = TestData.objects.get_or_create(testjob=job)
     if not created:
         # prevent updates of existing TestData
         logger.debug("[%s] skipping alteration of existing TestData", job.id)
