@@ -29,7 +29,10 @@ from django.conf import settings
 from django.core.exceptions import PermissionDenied
 from django.db.models import Count, Q
 from django.db import transaction
-from linaro_django_xmlrpc.models import ExposedAPI
+from linaro_django_xmlrpc.models import ExposedV2API
+
+from lava_common.compat import yaml_safe_dump, yaml_safe_load
+
 from lava_scheduler_app.models import (
     Device,
     DeviceType,
@@ -37,7 +40,6 @@ from lava_scheduler_app.models import (
     DevicesUnavailableException,
     TestJob,
 )
-from lava_scheduler_app.views import get_restricted_job
 from lava_scheduler_app.dbutils import (
     device_type_summary,
     testjob_submission,
@@ -50,10 +52,9 @@ from lava_scheduler_app.schema import (
 )
 
 # functions need to be members to be exposed in the API
-# pylint: disable=no-self-use
+
 
 # to make a function visible in the API, it must be a member of SchedulerAPI
-# pylint: disable=no-self-use
 
 
 def check_perm(perm):
@@ -89,7 +90,7 @@ def build_device_status_display(state, health):
         return "running"
 
 
-class SchedulerAPI(ExposedAPI):
+class SchedulerAPI(ExposedV2API):
     def submit_job(self, job_data):
         """
         Name
@@ -118,9 +119,10 @@ class SchedulerAPI(ExposedAPI):
             job = testjob_submission(job_data, self.user)
         except SubmissionException as exc:
             raise xmlrpc.client.Fault(400, "Problem with submitted job data: %s" % exc)
-        # FIXME: json error is not needed anymore
-        except (JSONDataError, JSONDecodeError, ValueError) as exc:
+        except ValueError as exc:
             raise xmlrpc.client.Fault(400, "Decoding job submission failed: %s." % exc)
+        except yaml.YAMLError as exc:
+            raise xmlrpc.client.Fault(400, "Invalid job definition: %s." % exc)
         except (Device.DoesNotExist, DeviceType.DoesNotExist):
             raise xmlrpc.client.Fault(404, "Specified device or device type not found.")
         except DevicesUnavailableException as exc:
@@ -153,9 +155,14 @@ class SchedulerAPI(ExposedAPI):
         """
         self._authenticate()
         try:
-            job = get_restricted_job(self.user, job_id)
+            job = TestJob.get_by_job_number(job_id)
         except TestJob.DoesNotExist:
-            raise xmlrpc.client.Fault(404, "Specified job not found.")
+            raise xmlrpc.client.Fault(404, "Job '%s' was not found." % job_id)
+
+        if not job.can_view(self.user):
+            raise xmlrpc.client.Fault(
+                403, "Job '%s' not available to user '%s'." % (job_id, self.user)
+            )
 
         if job.is_multinode:
             return self.submit_job(job.multinode_definition)
@@ -187,30 +194,19 @@ class SchedulerAPI(ExposedAPI):
 
         with transaction.atomic():
             try:
-                job = get_restricted_job(self.user, job_id, for_update=True)
-            except PermissionDenied:
-                raise xmlrpc.client.Fault(
-                    401, "Permission denied for user to job %s" % job_id
-                )
+                job = TestJob.get_by_job_number(job_id)
             except TestJob.DoesNotExist:
-                raise xmlrpc.client.Fault(404, "Specified job not found.")
+                raise xmlrpc.client.Fault(404, "Job '%s' was not found." % job_id)
 
-            if job.state in [TestJob.STATE_CANCELING, TestJob.STATE_FINISHED]:
-                # Don't do anything for jobs that ended already
-                return True
-            if not job.can_cancel(self.user):
-                raise xmlrpc.client.Fault(403, "Permission denied.")
-
-            if job.is_multinode:
-                multinode_jobs = TestJob.objects.select_for_update().filter(
-                    target_group=job.target_group
+            if not job.can_view(self.user):
+                raise xmlrpc.client.Fault(
+                    403, "Job '%s' not available to user '%s'." % (job_id, self.user)
                 )
-                for multinode_job in multinode_jobs:
-                    multinode_job.go_state_canceling()
-                    multinode_job.save()
-            else:
-                job.go_state_canceling()
-                job.save()
+
+            try:
+                job.cancel(self.user)
+            except PermissionDenied:
+                raise xmlrpc.client.Fault(403, "Permission denied.")
         return True
 
     def validate_yaml(self, yaml_string):
@@ -239,7 +235,7 @@ class SchedulerAPI(ExposedAPI):
         """
         try:
             # YAML can parse JSON as YAML, JSON cannot parse YAML at all
-            yaml_data = yaml.safe_load(yaml_string)
+            yaml_data = yaml_safe_load(yaml_string)
         except yaml.YAMLError as exc:
             raise xmlrpc.client.Fault(400, "Decoding job submission failed: %s." % exc)
         try:
@@ -271,17 +267,15 @@ class SchedulerAPI(ExposedAPI):
         This function returns an XML-RPC binary data of output file, provided
         the user is authenticated with an username and token.
         """
-        self._authenticate()
-        if not job_id:
-            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
         try:
-            job = get_restricted_job(self.user, job_id)
-        except PermissionDenied:
-            raise xmlrpc.client.Fault(
-                401, "Permission denied for user to job %s" % job_id
-            )
+            job = TestJob.get_by_job_number(job_id)
         except TestJob.DoesNotExist:
-            raise xmlrpc.client.Fault(404, "Specified job not found.")
+            raise xmlrpc.client.Fault(404, "Job '%s' was not found." % job_id)
+
+        if not job.can_view(self.user):
+            raise xmlrpc.client.Fault(
+                403, "Job '%s' not available to user '%s'." % (job_id, self.user)
+            )
 
         # Open the logs
         output_path = os.path.join(job.output_dir, "output.yaml")
@@ -793,131 +787,25 @@ class SchedulerAPI(ExposedAPI):
         is_public, _old_status, actual_device_id, definition, sub_id,
         requested_device_type_id, end_time, absolute_url, submitter_username
         """
-        self._authenticate()
-        if not job_id:
-            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
         try:
-            job = get_restricted_job(self.user, job_id)
-            job.status = job.get_legacy_status_display()
-            job.state = job.get_state_display()
-            job.health = job.get_health_display()
-            job.submitter_username = job.submitter.username
-            job.absolute_url = job.get_absolute_url()
-            job.is_pipeline = True
-        except PermissionDenied:
-            raise xmlrpc.client.Fault(
-                401, "Permission denied for user to job %s" % job_id
-            )
+            job = TestJob.get_by_job_number(job_id)
         except TestJob.DoesNotExist:
-            raise xmlrpc.client.Fault(404, "Specified job not found.")
+            raise xmlrpc.client.Fault(404, "Job '%s' was not found." % job_id)
+
+        if not job.can_view(self.user):
+            raise xmlrpc.client.Fault(
+                403, "Job '%s' not available to user '%s'." % (job_id, self.user)
+            )
+
+        job.status = job.get_legacy_status_display()
+        job.state = job.get_state_display()
+        job.health = job.get_health_display()
+        job.submitter_username = job.submitter.username
+        job.absolute_url = job.get_absolute_url()
+        job.is_pipeline = True
 
         return job
 
-    def job_status(self, job_id):
-        """
-        Name
-        ----
-        DEPRECATED - use `job_health` or `job_state` instead.
-
-        `job_status` (`job_id`)
-
-        Description
-        -----------
-        Get the status of given job id.
-
-        Arguments
-        ---------
-        `job_id`: string
-            Job id for which the output is required.
-
-        Return value
-        ------------
-        This function returns an XML-RPC structures of job status with the
-        following fields.
-        The user is authenticated with an username and token.
-
-        `job_status`: string
-        ['Submitted'|'Running'|'Complete'|'Incomplete'|'Canceled'|'Canceling']
-
-        `bundle_sha1`: string
-        The sha1 hash code of the bundle, if it existed. Otherwise it will be
-        an empty string. (LAVA V1 only)
-        """
-        self._authenticate()
-        if not job_id:
-            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
-        try:
-            job = get_restricted_job(self.user, job_id)
-        except PermissionDenied:
-            raise xmlrpc.client.Fault(
-                401, "Permission denied for user to job %s" % job_id
-            )
-        except TestJob.DoesNotExist:
-            raise xmlrpc.client.Fault(404, "Specified job not found.")
-
-        job_status = {"job_id": job.id}
-
-        if job.is_multinode:
-            job_status.update({"sub_id": job.sub_id})
-
-        job_status.update(
-            {"job_status": job.get_legacy_status_display(), "bundle_sha1": ""}
-        )
-        return job_status
-
-    def job_list_status(self, job_id_list):
-        """
-        Name
-        ----
-        DEPRECATED - use `job_health` or `job_state` instead.
-
-        job_list_status ([job_id, job_id, job_id])
-
-        Description
-        -----------
-        Get the status of a list of job ids.
-
-        Arguments
-        ---------
-        `job_id_list`: list
-            List of job ids for which the output is required.
-            For multinode jobs specify the job sub_id as a float
-            in the XML-RPC call:
-            job_list_status([1, 2, 3,1, 5])
-
-        Return value
-        ------------
-        The user needs to be authenticated with an username and token.
-
-        This function returns an XML-RPC structure of job status with the
-        following content.
-
-        `job_status`: string
-        {ID: ['Submitted'|'Running'|'Complete'|'Incomplete'|'Canceled'|'Canceling']}
-
-        If the user is not able to view one of the specified jobs, that entry
-        will be omitted.
-
-        """
-        self._authenticate()
-        job_status = {}
-        # optimise the query for a long list instead of using the
-        # convenience handlers
-        if not isinstance(job_id_list, list):
-            raise xmlrpc.client.Fault(400, "Bad request: needs to be a list")
-        if not all(isinstance(chk, (float, int)) for chk in job_id_list):
-            raise xmlrpc.client.Fault(
-                400, "Bad request: needs to be a list of integers or floats"
-            )
-        jobs = (
-            TestJob.objects.filter(Q(id__in=job_id_list) | Q(sub_id__in=job_id_list))
-            .visible_by_user(self.user)
-            .select_related("actual_device", "requested_device_type")
-        )
-        for job in jobs:
-            job_status[str(job.display_id)] = job.get_legacy_status_display()
-        return job_status
-
     def job_health(self, job_id):
         """
         Name
@@ -942,17 +830,15 @@ class SchedulerAPI(ExposedAPI):
         `job_health`: string
         ['Unknown'|'Complete'|'Incomplete'|'Canceled']
         """
-        self._authenticate()
-        if not job_id:
-            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
         try:
-            job = get_restricted_job(self.user, job_id)
-        except PermissionDenied:
-            raise xmlrpc.client.Fault(
-                401, "Permission denied for user to job %s" % job_id
-            )
+            job = TestJob.get_by_job_number(job_id)
         except TestJob.DoesNotExist:
-            raise xmlrpc.client.Fault(404, "Specified job not found.")
+            raise xmlrpc.client.Fault(404, "Job '%s' was not found." % job_id)
+
+        if not job.can_view(self.user):
+            raise xmlrpc.client.Fault(
+                403, "Job '%s' not available to user '%s'." % (job_id, self.user)
+            )
 
         job_health = {"job_id": job.id, "job_health": job.get_health_display()}
 
@@ -985,17 +871,15 @@ class SchedulerAPI(ExposedAPI):
         `job_state`: string
         ['Submitted'|'Scheduling'|'Scheduled'|'Running'|'Canceling'|'Finished']
         """
-        self._authenticate()
-        if not job_id:
-            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
         try:
-            job = get_restricted_job(self.user, job_id)
-        except PermissionDenied:
-            raise xmlrpc.client.Fault(
-                401, "Permission denied for user to job %s" % job_id
-            )
+            job = TestJob.get_by_job_number(job_id)
         except TestJob.DoesNotExist:
-            raise xmlrpc.client.Fault(404, "Specified job not found.")
+            raise xmlrpc.client.Fault(404, "Job '%s' was not found." % job_id)
+
+        if not job.can_view(self.user):
+            raise xmlrpc.client.Fault(
+                403, "Job '%s' not available to user '%s'." % (job_id, self.user)
+            )
 
         job_state = {"job_id": job.id, "job_state": job.get_state_display()}
 
@@ -1049,29 +933,6 @@ class SchedulerAPI(ExposedAPI):
 
         return jobs_list
 
-    def get_pipeline_device_config(self, device_hostname):
-        """
-        Name
-        ----
-        DEPRECATED - use `get_device_config` instead.
-
-        `get_pipeline_device_config` (`device_hostname`)
-
-        Description
-        -----------
-        Get the pipeline device configuration for given device hostname.
-
-        Arguments
-        ---------
-        `device_hostname`: string
-            Device hostname for which the configuration is required.
-
-        Return value
-        ------------
-        This function returns an XML-RPC binary data of output file.
-        """
-        return self.get_device_config(device_hostname)
-
     def get_device_config(self, device_hostname, context=None):
         """
         New in api_version 2 - see system.api_version()
@@ -1106,7 +967,7 @@ class SchedulerAPI(ExposedAPI):
         job_ctx = None
         if context is not None:
             try:
-                job_ctx = yaml.safe_load(context)
+                job_ctx = yaml_safe_load(context)
             except yaml.YAMLError as exc:
                 raise xmlrpc.client.Fault(
                     400, "Job context '%s' is not valid. %s" % (context, exc)
@@ -1124,7 +985,7 @@ class SchedulerAPI(ExposedAPI):
         config = device.load_configuration(job_ctx=job_ctx, output_format="yaml")
 
         # validate against the device schema
-        validate_device(yaml.safe_load(config))
+        validate_device(yaml_safe_load(config))
 
         return xmlrpc.client.Binary(config.encode("UTF-8"))
 
@@ -1281,12 +1142,12 @@ class SchedulerAPI(ExposedAPI):
                 continue
             try:
                 # validate against the device schema
-                validate_device(yaml.safe_load(config))
+                validate_device(yaml_safe_load(config))
             except SubmissionException as exc:
                 results[key] = {"Invalid": exc}
                 continue
             results[key] = {"Valid": None}
-        return xmlrpc.client.Binary(yaml.dump(results).encode("UTF-8"))
+        return xmlrpc.client.Binary(yaml_safe_dump(results).encode("UTF-8"))
 
     def get_publisher_event_socket(self):
         """
