@@ -18,13 +18,14 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
-import yaml
 import random
 from django.contrib.admin.models import LogEntry
 from django.utils.html import escape
 from django.utils.safestring import mark_safe
 from django.utils.timesince import timesince
 import django_tables2 as tables
+
+from lava_common.compat import yaml_dump
 from lava_results_app.models import TestCase
 from lava_scheduler_app.models import TestJob, Device, DeviceType, Worker
 from lava_server.lavatable import LavaTable
@@ -40,17 +41,13 @@ from django.utils import timezone
 # No function in this file is directly accessible via urls.py - those
 # functions need to go in views.py
 
-# pylint: disable=invalid-name
-
 
 class IDLinkColumn(tables.Column):
     def __init__(self, verbose_name="ID", **kw):
         kw["verbose_name"] = verbose_name
         super().__init__(**kw)
 
-    def render(
-        self, record, table=None
-    ):  # pylint: disable=arguments-differ,unused-argument
+    def render(self, record, table=None):
         return pklink(record)
 
 
@@ -237,15 +234,13 @@ class JobTable(LavaTable):
             return "Error"
         return retval
 
-    def render_description(self, value):  # pylint: disable=no-self-use
+    def render_description(self, value):
         if value:
             return value
         else:
             return ""
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         model = TestJob
         # alternatively, use 'fields' value to include specific fields.
         exclude = [
@@ -333,9 +328,7 @@ class IndexJobTable(JobTable):
                 '<span class="text-muted">Retired</span>'
             )
 
-    class Meta(
-        JobTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(JobTable.Meta):
         fields = (
             "id",
             "actions",
@@ -415,13 +408,11 @@ class FailedJobTable(JobTable):
             return ""
         action_metadata = failure.action_metadata
         if action_metadata is not None and "error_msg" in action_metadata:
-            return yaml.dump(failure.action_metadata["error_msg"])
+            return yaml_dump(failure.action_metadata["error_msg"])
         else:
             return ""
 
-    class Meta(
-        JobTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(JobTable.Meta):
         fields = ("id", "actions", "state", "device", "submit_time")
         sequence = ("id", "actions", "state", "device", "submit_time")
         exclude = ("submitter", "end_time", "priority", "description")
@@ -450,19 +441,17 @@ class LongestJobTable(JobTable):
     running = tables.Column(accessor="start_time", verbose_name="Running")
     running.orderable = False
 
-    def render_running(self, record):  # pylint: disable=no-self-use
+    def render_running(self, record):
         if not record.start_time:
             return ""
         return str(timezone.now() - record.start_time)
 
-    def render_device(self, record):  # pylint: disable=no-self-use
+    def render_device(self, record):
         if record.actual_device:
             return pklink(record.actual_device)
         return ""
 
-    class Meta(
-        JobTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(JobTable.Meta):
         fields = ("id", "actions", "state", "device")
         sequence = ("id", "actions", "state", "device")
         exclude = ("duration", "end_time")
@@ -482,9 +471,7 @@ class OverviewJobsTable(JobTable):
     submit_time = tables.DateColumn("Nd, g:ia")
     end_time = tables.DateColumn("Nd, g:ia")
 
-    class Meta(
-        JobTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(JobTable.Meta):
         fields = (
             "id",
             "actions",
@@ -512,9 +499,7 @@ class RecentJobsTable(JobTable):
     submit_time = tables.DateColumn("Nd, g:ia")
     end_time = tables.DateColumn("Nd, g:ia")
 
-    class Meta(
-        JobTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(JobTable.Meta):
         fields = (
             "id",
             "actions",
@@ -538,7 +523,7 @@ class RecentJobsTable(JobTable):
 
 
 class DeviceHealthTable(LavaTable):
-    def render_last_health_report_job(self, record):  # pylint: disable=no-self-use
+    def render_last_health_report_job(self, record):
         report = record.last_health_report_job
         if report is None:
             return ""
@@ -561,9 +546,7 @@ class DeviceHealthTable(LavaTable):
     )
     last_health_report_job = tables.Column("last report job")
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         sequence = [
             "hostname",
             "worker_host",
@@ -576,19 +559,19 @@ class DeviceHealthTable(LavaTable):
 
 
 class DeviceTypeTable(LavaTable):
-    def render_idle(self, record):  # pylint: disable=no-self-use
+    def render_idle(self, record):
         return record["idle"] if record["idle"] > 0 else ""
 
-    def render_offline(self, record):  # pylint: disable=no-self-use
+    def render_offline(self, record):
         return record["offline"] if record["offline"] > 0 else ""
 
-    def render_busy(self, record):  # pylint: disable=no-self-use
+    def render_busy(self, record):
         return record["busy"] if record["busy"] > 0 else ""
 
-    def render_name(self, record):  # pylint: disable=no-self-use
+    def render_name(self, record):
         return pklink(DeviceType.objects.get(name=record["device_type"]))
 
-    def render_queue(self, record):  # pylint: disable=no-self-use
+    def render_queue(self, record):
         count = TestJob.objects.filter(
             Q(state=TestJob.STATE_SUBMITTED),
             Q(requested_device_type=record["device_type"]),
@@ -604,9 +587,7 @@ class DeviceTypeTable(LavaTable):
     # sadly, this needs to be not orderable as it would otherwise sort by the accessor.
     queue = tables.Column(accessor="idle", verbose_name="Queue", orderable=False)
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         model = DeviceType
         exclude = [
             "display",
@@ -624,7 +605,7 @@ class DeviceTypeTable(LavaTable):
 
 
 class DeviceTable(LavaTable):
-    def render_device_type(self, record):  # pylint: disable=no-self-use
+    def render_device_type(self, record):
         return pklink(record.device_type)
 
     def render_worker_host(self, record):
@@ -690,9 +671,7 @@ class DeviceTable(LavaTable):
     health = tables.Column(verbose_name="Health")
     tags = TagsColumn()
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         model = Device
         exclude = [
             "device_version",
@@ -712,7 +691,7 @@ class DeviceTable(LavaTable):
         }
 
 
-class WorkerTable(LavaTable):  # pylint: disable=too-few-public-methods,no-init
+class WorkerTable(LavaTable):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.show_help = True
@@ -722,6 +701,8 @@ class WorkerTable(LavaTable):  # pylint: disable=too-few-public-methods,no-init
     <a href="{{ record.get_absolute_url }}">{{ record.hostname }}</a>
     """
     )
+    description = tables.Column(accessor="description", verbose_name="Description")
+    version = tables.Column(accessor="version", verbose_name="Version")
 
     def render_state(self, record):
         if record.state == Worker.STATE_ONLINE:
@@ -760,9 +741,7 @@ class WorkerTable(LavaTable):  # pylint: disable=too-few-public-methods,no-init
     def render_last_ping(self, record):
         return timesince(record.last_ping)
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         model = Worker
         sequence = ["hostname", "state", "health", "description"]
 
@@ -801,9 +780,7 @@ class DeviceLogEntryTable(LogEntryTable):
 
 
 class NoWorkerDeviceTable(DeviceTable):
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         exclude = [
             "worker_host",
             "device_version",
@@ -816,16 +793,14 @@ class NoWorkerDeviceTable(DeviceTable):
         queries = {"device_state_query": "state", "device_health_query": "health"}
 
 
-class HealthJobSummaryTable(tables.Table):  # pylint: disable=too-few-public-methods
+class HealthJobSummaryTable(tables.Table):
 
     length = 10
     Duration = tables.Column()
     Complete = tables.Column()
     Failed = tables.Column()
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         model = None
 
 
@@ -837,7 +812,7 @@ class QueueJobsTable(JobTable):
         template_name="lava_scheduler_app/job_actions_field.html"
     )
 
-    def render_requested_device_type(self, record):  # pylint: disable=no-self-use
+    def render_requested_device_type(self, record):
         return mark_safe(  # nosec - internal data
             '<a href="%s" title="%s device_type">%s</a>'
             % (
@@ -889,10 +864,10 @@ class QueueJobsTable(JobTable):
 
 
 class PassingHealthTable(DeviceHealthTable):
-    def render_device_type(self, record):  # pylint: disable=no-self-use
+    def render_device_type(self, record):
         return pklink(record.device_type)
 
-    def render_last_health_report_job(self, record):  # pylint: disable=no-self-use
+    def render_last_health_report_job(self, record):
         report = record.last_health_report_job
         return mark_safe(  # nosec - internal data
             '<a href="%s">%s</a>' % (report.get_absolute_url(), report)
@@ -900,9 +875,7 @@ class PassingHealthTable(DeviceHealthTable):
 
     device_type = tables.Column()
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         exclude = ["worker_host", "last_report_time"]
         sequence = ["hostname", "device_type", "health", "last_health_report_job"]
         searches = {"hostname": "contains"}
@@ -919,7 +892,7 @@ class RunningTable(LavaTable):
 
     # deprecated: dynamic connections are TestJob without a device
 
-    def render_jobs(self, record):  # pylint: disable=no-self-use
+    def render_jobs(self, record):
         count = TestJob.objects.filter(
             Q(state=TestJob.STATE_RUNNING),
             Q(requested_device_type=record.name)
@@ -927,13 +900,13 @@ class RunningTable(LavaTable):
         ).count()
         return count if count > 0 else ""
 
-    def render_reserved(self, record):  # pylint: disable=no-self-use
+    def render_reserved(self, record):
         count = Device.objects.filter(
             device_type=record.name, state=Device.STATE_RESERVED
         ).count()
         return count if count > 0 else ""
 
-    def render_running(self, record):  # pylint: disable=no-self-use
+    def render_running(self, record):
         count = Device.objects.filter(
             device_type=record.name, state=Device.STATE_RUNNING
         ).count()
@@ -947,9 +920,7 @@ class RunningTable(LavaTable):
     running = tables.Column(accessor="display", orderable=False, verbose_name="Running")
     jobs = tables.Column(accessor="display", orderable=False, verbose_name="Jobs")
 
-    class Meta(
-        LavaTable.Meta
-    ):  # pylint: disable=too-few-public-methods,no-init,no-self-use
+    class Meta(LavaTable.Meta):
         model = DeviceType
         sequence = ["name", "reserved", "running", "jobs"]
         exclude = [
