@@ -22,6 +22,7 @@ import copy
 import errno
 import os
 import subprocess  # nosec verified
+import warnings
 
 from django.conf import settings
 
@@ -60,9 +61,7 @@ def _split_multinode_vland(submission, jobs):
     return jobs
 
 
-def split_multinode_yaml(
-    submission, target_group
-):  # pylint: disable=too-many-branches,too-many-locals,too-many-statements
+def split_multinode_yaml(submission, target_group):
     """
     Handles the lava-multinode protocol requirements.
     Uses the multinode protocol requirements to generate as many YAML
@@ -318,6 +317,27 @@ def get_ldap_user_properties(ldap_user):
                     user_properties["given_name"] = result_data.get(
                         "givenName", [None]
                     )[0]
-                    return user_properties
+                    return {k: v.decode() for (k, v) in user_properties.items()}
             except ldap.NO_SUCH_OBJECT:
                 raise
+
+
+def get_encryption_settings(options):
+    # Get encryption settings from command line arguments. If not present,
+    # fall back to django settings.
+    encryption_settings = {
+        "encrypt": settings.ENCRYPT,
+        "master_cert": settings.MASTER_CERT,
+        "slaves_certs": settings.SLAVES_CERTS,
+    }
+
+    if options["encrypt"]:
+        warnings.warn(
+            "Using encryption options in command line is deprecated. Switch to django settings module for encryption configuration.",
+            DeprecationWarning,
+        )
+        encryption_settings["encrypt"] = True
+        encryption_settings["master_cert"] = options["master_cert"]
+        encryption_settings["slaves_certs"] = options["slaves_certs"]
+
+    return encryption_settings
