@@ -18,7 +18,6 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
-# pylint: disable=too-many-lines,invalid-name
 
 import contextlib
 import datetime
@@ -62,6 +61,7 @@ from lava_common.schemas import validate
 
 from lava_server.views import index as lava_index
 from lava_server.bread_crumbs import BreadCrumb, BreadCrumbTrail
+from lava_server.compat import djt2_paginator_class
 
 from lava_scheduler_app.models import (
     Device,
@@ -69,8 +69,6 @@ from lava_scheduler_app.models import (
     Tag,
     TestJob,
     TestJobUser,
-    JSONDataError,
-    DevicesUnavailableException,
     Worker,
 )
 from lava_scheduler_app.dbutils import (
@@ -80,7 +78,7 @@ from lava_scheduler_app.dbutils import (
     testjob_submission,
     validate_job,
 )
-from lava_scheduler_app.logutils import open_logs, size_logs, read_logs
+from lava_scheduler_app.logutils import logs_instance
 from lava_scheduler_app.templatetags.utils import udecode
 
 from lava_server.lavatable import LavaView
@@ -120,9 +118,6 @@ from lava_scheduler_app.tables import (
     RunningTable,
 )
 
-# pylint: disable=too-many-ancestors,too-many-arguments,too-many-locals
-# pylint: disable=too-many-statements,too-many-branches,too-many-return-statements
-# pylint: disable=no-self-use,too-many-nested-blocks,too-few-public-methods
 
 # The only functions which need to go in this file are those directly
 # referenced in urls.py - other support functions can go in tables.py or similar.
@@ -133,7 +128,7 @@ def _str_to_bool(string):
 
 
 class JobTableView(LavaView):
-    def device_query(self, term):  # pylint: disable=no-self-use
+    def device_query(self, term):
         device = list(
             Device.objects.filter(hostname__contains=term).visible_by_user(
                 self.request.user
@@ -187,6 +182,8 @@ class FailureTableView(JobTableView):
         jobs = visible_jobs_with_custom_sort(self.request.user).filter(
             health__in=failures
         )
+        jobs = jobs.select_related("actual_device")
+        jobs = jobs.prefetch_related("failure_tags")
 
         health = self.request.GET.get("health_check")
         if health:
@@ -308,6 +305,7 @@ class DeviceTableView(JobTableView):
             .prefetch_related("tags")
             .visible_by_user(self.request.user)
             .order_by("hostname")
+            .distinct()
         )
 
 
@@ -316,7 +314,11 @@ class JobErrorsView(LavaView):
         q = TestCase.objects.filter(
             suite__name="lava", result=TestCase.RESULT_FAIL
         ).visible_by_user(self.request.user)
-        q = q.filter(metadata__regex="error_type: (Configuration|Infrastructure|Bug)")
+        q = q.filter(
+            Q(metadata__contains="error_type: Configuration")
+            | Q(metadata__contains="error_type: Infrastructure")
+            | Q(metadata__contains="error_type: Bug")
+        )
         q = q.select_related("suite", "suite__job__actual_device")
         return q.order_by("-suite__job__id")
 
@@ -391,82 +393,48 @@ def workers(request):
     )
 
 
-def type_report_data(start_day, end_day, dt, health_check):
+def report_data(start_day, end_day, devices, url_param):
     now = timezone.now()
     start_date = now + datetime.timedelta(start_day)
     end_date = now + datetime.timedelta(end_day)
 
-    res = TestJob.objects.filter(
-        actual_device__in=Device.objects.filter(device_type=dt),
-        health_check=health_check,
-        start_time__range=(start_date, end_date),
-    )
-    url = reverse("lava.scheduler.failure_report")
-    params = "start=%s&end=%s&device_type=%s&health_check=%d" % (
-        start_day,
-        end_day,
-        dt,
-        health_check,
-    )
-    return {
-        "pass": res.filter(health=TestJob.HEALTH_COMPLETE).count(),
-        "fail": res.filter(
-            health__in=[TestJob.HEALTH_CANCELED, TestJob.HEALTH_INCOMPLETE]
-        ).count(),
-        "date": start_date.strftime("%m-%d"),
-        "failure_url": "%s?%s" % (url, params),
-    }
+    res = TestJob.objects.filter(state=TestJob.STATE_FINISHED)
+    res = res.filter(start_time__range=(start_date, end_date))
+    if devices is not None:
+        res = res.filter(actual_device__in=devices)
 
-
-def device_report_data(start_day, end_day, device, health_check):
-    now = timezone.now()
-    start_date = now + datetime.timedelta(start_day)
-    end_date = now + datetime.timedelta(end_day)
-
-    res = TestJob.objects.filter(
-        actual_device=device,
-        health_check=health_check,
-        start_time__range=(start_date, end_date),
-    )
-    url = reverse("lava.scheduler.failure_report")
-    params = "start=%s&end=%s&device=%s&health_check=%d" % (
-        start_day,
-        end_day,
-        device.pk,
-        health_check,
-    )
-    return {
-        "pass": res.filter(health=TestJob.HEALTH_COMPLETE).count(),
-        "fail": res.filter(
-            health__in=[TestJob.HEALTH_CANCELED, TestJob.HEALTH_INCOMPLETE]
-        ).count(),
-        "date": start_date.strftime("%m-%d"),
-        "failure_url": "%s?%s" % (url, params),
-    }
-
-
-def job_report(start_day, end_day, health_check):
-    now = timezone.now()
-    start_date = now + datetime.timedelta(start_day)
-    end_date = now + datetime.timedelta(end_day)
-
-    res = TestJob.objects.filter(
-        health_check=health_check, start_time__range=(start_date, end_date)
-    )
-    res = res.filter(state=TestJob.STATE_FINISHED)
-    res = res.values("health")
+    res = res.values("health", "health_check")
     res = res.aggregate(
-        passing=Sum(
+        health_pass=Sum(
             Case(
-                When(health=TestJob.HEALTH_COMPLETE, then=1),
+                When(health=TestJob.HEALTH_COMPLETE, health_check=True, then=1),
                 default=0,
                 output_field=IntegerField(),
             )
         ),
-        failing=Sum(
+        job_pass=Sum(
+            Case(
+                When(health=TestJob.HEALTH_COMPLETE, health_check=False, then=1),
+                default=0,
+                output_field=IntegerField(),
+            )
+        ),
+        health_fail=Sum(
             Case(
                 When(
                     health__in=[TestJob.HEALTH_CANCELED, TestJob.HEALTH_INCOMPLETE],
+                    health_check=True,
+                    then=1,
+                ),
+                default=0,
+                output_field=IntegerField(),
+            )
+        ),
+        job_fail=Sum(
+            Case(
+                When(
+                    health__in=[TestJob.HEALTH_CANCELED, TestJob.HEALTH_INCOMPLETE],
+                    health_check=False,
                     then=1,
                 ),
                 default=0,
@@ -476,13 +444,34 @@ def job_report(start_day, end_day, health_check):
     )
 
     url = reverse("lava.scheduler.failure_report")
-    params = "start=%s&end=%s&health_check=%d" % (start_day, end_day, health_check)
-    return {
-        "pass": res["passing"] or 0,
-        "fail": res["failing"] or 0,
-        "date": start_date.strftime("%m-%d"),
-        "failure_url": "%s?%s" % (url, params),
-    }
+    params = "start=%s&end=%s%s" % (start_day, end_day, url_param)
+    return (
+        {
+            "pass": res["health_pass"] or 0,
+            "fail": res["health_fail"] or 0,
+            "date": start_date.strftime("%m-%d"),
+            "failure_url": "%s?%s&health_check=1" % (url, params),
+        },
+        {
+            "pass": res["job_pass"] or 0,
+            "fail": res["job_fail"] or 0,
+            "date": start_date.strftime("%m-%d"),
+            "failure_url": "%s?%s&health_check=0" % (url, params),
+        },
+    )
+
+
+def type_report_data(start_day, end_day, dt):
+    devices = Device.objects.filter(device_type=dt)
+    return report_data(start_day, end_day, devices, f"&device_type={dt}")
+
+
+def device_report_data(start_day, end_day, device):
+    return report_data(start_day, end_day, [device], f"&device={device}")
+
+
+def job_report_data(start_day, end_day):
+    return report_data(start_day, end_day, None, "")
 
 
 @BreadCrumb("Reports", parent=index)
@@ -492,11 +481,15 @@ def reports(request):
     job_day_report = []
     job_week_report = []
     for day in reversed(range(7)):
-        health_day_report.append(job_report(day * -1 - 1, day * -1, True))
-        job_day_report.append(job_report(day * -1 - 1, day * -1, False))
+        data = job_report_data(day * -1 - 1, day * -1)
+        health_day_report.append(data[0])
+        job_day_report.append(data[1])
+
     for week in reversed(range(10)):
-        health_week_report.append(job_report(week * -7 - 7, week * -7, True))
-        job_week_report.append(job_report(week * -7 - 7, week * -7, False))
+        data = job_report_data(week * -7 - 7, week * -7)
+        health_week_report.append(data[0])
+        job_week_report.append(data[1])
+
     return render(
         request,
         "lava_scheduler_app/reports.html",
@@ -512,7 +505,6 @@ def reports(request):
 
 @BreadCrumb("Failure Report", parent=reports)
 def failure_report(request):
-
     data = FailureTableView(request)
     ptable = FailedJobTable(data.get_table_data(), request=request)
     RequestConfig(request, paginate={"per_page": ptable.length}).configure(ptable)
@@ -723,6 +715,7 @@ class NoDTDeviceView(DeviceTableView):
         return (
             Device.objects.exclude(health=Device.HEALTH_RETIRED)
             .visible_by_user(self.request.user)
+            .select_related("device_type", "worker_host")
             .order_by("hostname")
         )
 
@@ -785,42 +778,36 @@ def device_type_detail(request, pk):
         actual_device__in=devices,
         health_check=True,
         submit_time__gte=(now - datetime.timedelta(days=1)),
-        submit_time__lt=now,
         health=TestJob.HEALTH_COMPLETE,
     ).count()
     daily_failed = TestJob.objects.filter(
         actual_device__in=devices,
         health_check=True,
         submit_time__gte=(now - datetime.timedelta(days=1)),
-        submit_time__lt=now,
         health__in=[TestJob.HEALTH_CANCELED, TestJob.HEALTH_INCOMPLETE],
     ).count()
     weekly_complete = TestJob.objects.filter(
         actual_device__in=devices,
         health_check=True,
         submit_time__gte=(now - datetime.timedelta(days=7)),
-        submit_time__lt=now,
         health=TestJob.HEALTH_COMPLETE,
     ).count()
     weekly_failed = TestJob.objects.filter(
         actual_device__in=devices,
         health_check=True,
         submit_time__gte=(now - datetime.timedelta(days=7)),
-        submit_time__lt=now,
         health__in=[TestJob.HEALTH_CANCELED, TestJob.HEALTH_INCOMPLETE],
     ).count()
     monthly_complete = TestJob.objects.filter(
         actual_device__in=devices,
         health_check=True,
         submit_time__gte=(now - datetime.timedelta(days=30)),
-        submit_time__lt=now,
         health=TestJob.HEALTH_COMPLETE,
     ).count()
     monthly_failed = TestJob.objects.filter(
         actual_device__in=devices,
         health_check=True,
         submit_time__gte=(now - datetime.timedelta(days=30)),
-        submit_time__lt=now,
         health__in=[TestJob.HEALTH_CANCELED, TestJob.HEALTH_INCOMPLETE],
     ).count()
     health_summary_data = [
@@ -968,24 +955,22 @@ def device_type_health_history_log(request, pk):
 @BreadCrumb("Report", parent=device_type_detail, needs=["pk"])
 def device_type_reports(request, pk):
     device_type = get_object_or_404(DeviceType, pk=pk)
+    if not device_type.can_view(request.user):
+        raise PermissionDenied()
+
     health_day_report = []
     health_week_report = []
     job_day_report = []
     job_week_report = []
     for day in reversed(range(7)):
-        health_day_report.append(
-            type_report_data(day * -1 - 1, day * -1, device_type, True)
-        )
-        job_day_report.append(
-            type_report_data(day * -1 - 1, day * -1, device_type, False)
-        )
+        data = type_report_data(day * -1 - 1, day * -1, device_type)
+        health_day_report.append(data[0])
+        job_day_report.append(data[1])
+
     for week in reversed(range(10)):
-        health_week_report.append(
-            type_report_data(week * -7 - 7, week * -7, device_type, True)
-        )
-        job_week_report.append(
-            type_report_data(week * -7 - 7, week * -7, device_type, False)
-        )
+        data = type_report_data(week * -7 - 7, week * -7, device_type)
+        health_week_report.append(data[0])
+        job_week_report.append(data[1])
 
     long_running = (
         TestJob.objects.filter(
@@ -1088,7 +1073,9 @@ class FavoriteJobsView(JobTableView):
 
 class AllJobsView(JobTableView):
     def get_queryset(self):
-        return visible_jobs_with_custom_sort(self.request.user)
+        return visible_jobs_with_custom_sort(self.request.user).select_related(
+            "actual_device", "requested_device_type", "submitter"
+        )
 
 
 @BreadCrumb("Jobs", parent=index)
@@ -1115,9 +1102,11 @@ def job_list(request):
 def job_errors(request):
     data = JobErrorsView(request, model=TestCase, table_class=JobErrorsTable)
     ptable = JobErrorsTable(
-        data.get_table_data(), request=request, prefix="job_errors_"
+        data.get_table_data(), prefix="job_errors_", template_name="lazytables.html"
     )
-    RequestConfig(request, paginate={"per_page": ptable.length}).configure(ptable)
+    RequestConfig(
+        request, paginate={"per_page": ptable.length, **djt2_paginator_class()}
+    ).configure(ptable)
     return render(
         request,
         "lava_scheduler_app/job_errors.html",
@@ -1167,6 +1156,7 @@ def job_submit(request):
             warnings = ""
             errors = ""
             try:
+                validate_job(request.POST.get("definition-input"))
                 try:
                     validate(
                         yaml_safe_load(request.POST.get("definition-input")),
@@ -1289,12 +1279,12 @@ def job_detail(request, pk):
     }
 
     try:
-        job_file_size = size_logs(job.output_dir)
+        job_file_size = logs_instance.size(job)
         if job_file_size is not None and job_file_size >= job.size_limit:
             log_data = []
             data["size_warning"] = True
         else:
-            log_data = read_logs(job.output_dir)
+            log_data = logs_instance.read(job)
             log_data = yaml_load(log_data)
     except OSError:
         log_data = []
@@ -1508,7 +1498,7 @@ def job_status(request, pk):
         response_dict["actual_device"] = html
 
     if job.start_time:
-        response_dict["started"] = naturaltime(job.start_time)
+        response_dict["started"] = str(naturaltime(job.start_time))
         response_dict["duration"] = timeuntil(timezone.now(), job.start_time)
 
     if job.state != job.STATE_FINISHED:
@@ -1538,7 +1528,7 @@ def job_status(request, pk):
 def job_timing(request, pk):
     job = get_restricted_job(request.user, pk, request=request)
     try:
-        data = read_logs(job.output_dir)
+        data = logs_instance.read(job)
         logs = yaml_load(data)
     except OSError:
         raise Http404
@@ -1660,7 +1650,7 @@ def job_configuration(request, pk):
 def job_log_file_plain(request, pk):
     job = get_restricted_job(request.user, pk, request=request)
     try:
-        data = open_logs(job.output_dir)
+        data = logs_instance.open(job)
         response = FileResponse(data, content_type="application/yaml")
         response["Content-Disposition"] = "attachment; filename=job_%d.log" % job.id
         return response
@@ -1676,14 +1666,14 @@ def job_log_incremental(request, pk):
     except ValueError:
         first_line = 0
 
-    job_file_size = size_logs(job.output_dir)
+    job_file_size = logs_instance.size(job)
     if job_file_size is not None and job_file_size >= job.size_limit:
         response = HttpResponse(simplejson.dumps([]), content_type="application/json")
         response["X-Size-Warning"] = "1"
         return response
 
     try:
-        data = read_logs(job.output_dir, first_line)
+        data = logs_instance.read(job, first_line)
         data = yaml_load(data)
         # When reaching EOF, yaml.load does return None instead of []
         if not data:
@@ -1711,25 +1701,16 @@ def job_log_incremental(request, pk):
     return response
 
 
+@transaction.atomic
 def job_cancel(request, pk):
-    with transaction.atomic():
-        job = get_restricted_job(request.user, pk, request=request, for_update=True)
-        if job.can_cancel(request.user):
-            if job.is_multinode:
-                multinode_jobs = TestJob.objects.select_for_update().filter(
-                    target_group=job.target_group
-                )
-                for multinode_job in multinode_jobs:
-                    multinode_job.go_state_canceling()
-                    multinode_job.save()
-            else:
-                job.go_state_canceling()
-                job.save()
-            return redirect(job)
-        else:
-            return HttpResponseForbidden(
-                "you cannot cancel this job", content_type="text/plain"
-            )
+    job = get_restricted_job(request.user, pk, request=request, for_update=True)
+    try:
+        job.cancel(request.user)
+        return redirect(job)
+    except PermissionDenied:
+        return HttpResponseForbidden(
+            "you cannot cancel this job", content_type="text/plain"
+        )
 
 
 def job_fail(request, pk):
@@ -1764,7 +1745,6 @@ def job_resubmit(request, pk):
 
         if is_resubmit:
             try:
-
                 original = job
                 job = testjob_submission(
                     request.POST.get("definition-input"),
@@ -1816,17 +1796,8 @@ def job_resubmit(request, pk):
             else:
                 definition = job.display_definition
 
-            try:
-                response_data["definition_input"] = definition
-                return render(
-                    request, "lava_scheduler_app/job_submit.html", response_data
-                )
-            except (JSONDataError, ValueError, DevicesUnavailableException) as e:
-                response_data["error"] = str(e)
-                response_data["definition_input"] = definition
-                return render(
-                    request, "lava_scheduler_app/job_submit.html", response_data
-                )
+            response_data["definition_input"] = definition
+            return render(request, "lava_scheduler_app/job_submit.html", response_data)
 
     else:
         return HttpResponseForbidden(
@@ -1890,8 +1861,10 @@ class RecentJobsView(JobTableView):
         self.device = device
 
     def get_queryset(self):
-        return visible_jobs_with_custom_sort(self.request.user).filter(
-            actual_device=self.device
+        return (
+            visible_jobs_with_custom_sort(self.request.user)
+            .filter(actual_device=self.device)
+            .select_related("submitter")
         )
 
 
@@ -2016,22 +1989,20 @@ def device_reports(request, pk):
     device = get_object_or_404(Device, pk=pk)
     if not device.can_view(request.user):
         raise PermissionDenied()
+
     health_day_report = []
     health_week_report = []
     job_day_report = []
     job_week_report = []
     for day in reversed(range(7)):
-        health_day_report.append(
-            device_report_data(day * -1 - 1, day * -1, device, True)
-        )
-        job_day_report.append(device_report_data(day * -1 - 1, day * -1, device, False))
+        data = device_report_data(day * -1 - 1, day * -1, device)
+        health_day_report.append(data[0])
+        job_day_report.append(data[1])
+
     for week in reversed(range(10)):
-        health_week_report.append(
-            device_report_data(week * -7 - 7, week * -7, device, True)
-        )
-        job_week_report.append(
-            device_report_data(week * -7 - 7, week * -7, device, False)
-        )
+        data = device_report_data(week * -7 - 7, week * -7, device)
+        health_week_report.append(data[0])
+        job_week_report.append(data[1])
 
     long_running = (
         TestJob.objects.filter(
@@ -2154,6 +2125,9 @@ def worker_health(request, pk):
 
 def username_list_json(request):
 
+    if not request.user.is_authenticated:
+        raise PermissionDenied()
+
     term = request.GET["term"]
     users = []
     for user in User.objects.filter(Q(username__istartswith=term)):
@@ -2163,8 +2137,10 @@ def username_list_json(request):
 
 class HealthCheckJobsView(JobTableView):
     def get_queryset(self):
-        return visible_jobs_with_custom_sort(self.request.user).filter(
-            health_check=True
+        return (
+            visible_jobs_with_custom_sort(self.request.user)
+            .filter(health_check=True)
+            .select_related("actual_device", "requested_device_type", "submitter")
         )
 
 
@@ -2273,8 +2249,6 @@ def download_device_type_template(request, pk):
 def similar_jobs(request, pk):
     logger = logging.getLogger("lava_scheduler_app")
     job = get_restricted_job(request.user, pk, request=request)
-    if not job.can_change_priority(request.user):
-        raise PermissionDenied()
 
     entity = ContentType.objects.get_for_model(TestJob).model
 
