@@ -17,9 +17,6 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
-# pylint: disable=wrong-import-order,bad-continuation
-
-import contextlib
 import logging
 import os
 import time
@@ -32,13 +29,14 @@ from django.conf import settings
 from django.db import connection, transaction
 from django.db.utils import DatabaseError, InterfaceError, OperationalError
 
-from lava_common.compat import yaml_load
+from lava_common.compat import yaml_load, yaml_dump
+from lava_common.version import __version__
 from lava_results_app.models import TestCase
 from lava_server.cmdutils import LAVADaemonCommand, watch_directory
 from lava_scheduler_app.models import TestJob
 from lava_scheduler_app.signals import send_event
-from lava_scheduler_app.utils import mkdir
-from lava_scheduler_app.logutils import line_count, write_logs
+from lava_scheduler_app.utils import mkdir, get_encryption_settings
+from lava_scheduler_app.logutils import logs_instance
 from lava_results_app.dbutils import map_scanned_results, create_metadata_store
 
 
@@ -49,19 +47,21 @@ BULK_CREATE_TIMEOUT = 10
 FD_TIMEOUT = 60
 
 
-class JobHandler:  # pylint: disable=too-few-public-methods
+class JobHandler:
     def __init__(self, job):
-        self.output_dir = job.output_dir
-        self.output = open(os.path.join(self.output_dir, "output.yaml"), "ab")
-        self.index = open(os.path.join(self.output_dir, "output.idx"), "ab")
+        self.job = job
+        self.output = open(os.path.join(job.output_dir, "output.yaml"), "ab")
+        self.index = open(os.path.join(job.output_dir, "output.idx"), "ab")
         self.last_usage = time.time()
         self.markers = {}
 
     def write(self, message):
-        write_logs(self.output, self.index, (message + "\n").encode("utf-8"))
+        logs_instance.write(
+            self.job, (message + "\n").encode("utf-8"), self.output, self.index
+        )
 
     def line_count(self):
-        return line_count(self.index)
+        return logs_instance.line_count(self.job)
 
     def close(self):
         self.index.close()
@@ -129,6 +129,9 @@ class Command(LAVADaemonCommand):
         # Initialize logging.
         self.setup_logging("lava-logs", options["level"], options["log_file"], FORMAT)
 
+        self.logger.info("[INIT] Starting lava-logs")
+        self.logger.info("[INIT] Version %s", __version__)
+
         self.logger.info("[INIT] Dropping privileges")
         if not self.drop_privileges(options["user"], options["group"]):
             self.logger.error("[INIT] Unable to drop privileges")
@@ -137,7 +140,7 @@ class Command(LAVADaemonCommand):
         filename = os.path.join(settings.MEDIA_ROOT, "lava-logs-config.yaml")
         self.logger.debug("[INIT] Dumping config to %s", filename)
         with open(filename, "w") as output:
-            yaml.dump(options, output)
+            yaml_dump(options, output)
 
         # Create the sockets
         context = zmq.Context()
@@ -155,21 +158,26 @@ class Command(LAVADaemonCommand):
             self.log_socket.setsockopt(zmq.IPV6, 1)
             self.controler.setsockopt(zmq.IPV6, 1)
 
-        if options["encrypt"]:
+        encryption_settings = get_encryption_settings(options)
+        if encryption_settings["encrypt"]:
             self.logger.info("[INIT] Starting encryption")
             try:
                 self.auth = ThreadAuthenticator(context)
                 self.auth.start()
                 self.logger.debug(
-                    "[INIT] Opening master certificate: %s", options["master_cert"]
+                    "[INIT] Opening master certificate: %s",
+                    encryption_settings["master_cert"],
                 )
                 master_public, master_secret = zmq.auth.load_certificate(
-                    options["master_cert"]
+                    encryption_settings["master_cert"]
                 )
                 self.logger.debug(
-                    "[INIT] Using slaves certificates from: %s", options["slaves_certs"]
+                    "[INIT] Using slaves certificates from: %s",
+                    encryption_settings["slaves_certs"],
+                )
+                self.auth.configure_curve(
+                    domain="*", location=encryption_settings["slaves_certs"]
                 )
-                self.auth.configure_curve(domain="*", location=options["slaves_certs"])
             except OSError as err:
                 self.logger.error("[INIT] %s", err)
                 self.auth.stop()
@@ -181,9 +189,9 @@ class Command(LAVADaemonCommand):
             self.controler.curve_secretkey = master_secret
             self.controler.curve_serverkey = master_public
 
-        self.logger.debug("[INIT] Watching %s", options["slaves_certs"])
-        self.cert_dir_path = options["slaves_certs"]
-        self.inotify_fd = watch_directory(options["slaves_certs"])
+        self.logger.debug("[INIT] Watching %s", encryption_settings["slaves_certs"])
+        self.cert_dir_path = encryption_settings["slaves_certs"]
+        self.inotify_fd = watch_directory(encryption_settings["slaves_certs"])
         if self.inotify_fd is None:
             self.logger.error("[INIT] Unable to start inotify")
 
@@ -237,7 +245,7 @@ class Command(LAVADaemonCommand):
             self.flush_test_cases()
             self.logger.info("[EXIT] Closing the logging socket: the queue is empty")
             self.log_socket.close()
-            if options["encrypt"]:
+            if encryption_settings["encrypt"]:
                 self.auth.stop()
             context.term()
 
@@ -250,17 +258,18 @@ class Command(LAVADaemonCommand):
             TestCase.objects.bulk_create(self.test_cases)
             self.logger.info("Saving %d test cases", len(self.test_cases))
             self.test_cases = []
-        except DatabaseError as exc:
-            self.logger.error("Unable to flush the test cases")
-            self.logger.exception(exc)
+        except (DatabaseError, ValueError):
+            self.logger.warning("Unable to flush the test cases")
             self.logger.warning(
                 "Saving test cases one by one and dropping the faulty ones"
             )
             saved = 0
             for tc in self.test_cases:
-                with contextlib.suppress(DatabaseError):
+                try:
                     tc.save()
                     saved += 1
+                except (DatabaseError, ValueError) as exc:
+                    self.logger.error("Droping %s: %s", tc, str(exc))
             self.logger.info(
                 "%d test cases saved, %d dropped", saved, len(self.test_cases) - saved
             )
@@ -331,10 +340,16 @@ class Command(LAVADaemonCommand):
             # Inotify socket
             if sockets.get(self.inotify_fd) == zmq.POLLIN:
                 os.read(self.inotify_fd, 4096)
-                self.logger.debug(
-                    "[AUTH] Reloading certificates from %s", self.cert_dir_path
-                )
-                self.auth.configure_curve(domain="*", location=self.cert_dir_path)
+                if self.auth is not None:
+                    self.logger.debug(
+                        "[AUTH] Reloading certificates from %s", self.cert_dir_path
+                    )
+                    self.auth.configure_curve(domain="*", location=self.cert_dir_path)
+                else:
+                    self.logger.error(
+                        "[AUTH] New certificates in %s but encryption is disabled",
+                        self.cert_dir_path,
+                    )
 
             # Nothing received
             else:
@@ -348,9 +363,10 @@ class Command(LAVADaemonCommand):
     def logging_socket(self):
         msg = self.log_socket.recv_multipart()
         try:
-            (job_id, message) = (
-                u(m) for m in msg
-            )  # pylint: disable=unbalanced-tuple-unpacking
+            (job_id, message) = (u(m) for m in msg)
+        except UnicodeDecodeError:
+            self.logger.error("[POLL] Invalid log message: can't be decoded")
+            return
         except ValueError:
             # do not let a bad message stop the master.
             self.logger.error("[POLL] failed to parse log message, skipping: %s", msg)
@@ -404,17 +420,9 @@ class Command(LAVADaemonCommand):
             if m_type is None or case is None:
                 self.logger.error("[%s] invalid marker: %s", job_id, message_msg)
                 return
-            self.jobs[job_id].markers.setdefault(case, {})[m_type] = self.jobs[
-                job_id
-            ].line_count()
             # This is in fact the previous line
-            self.jobs[job_id].markers[case][m_type] -= 1
-            self.logger.debug(
-                "[%s] marker: %s line: %s",
-                job_id,
-                message_msg,
-                self.jobs[job_id].markers[case][m_type],
-            )
+            line = self.jobs[job_id].line_count() - 1
+            self.jobs[job_id].markers.setdefault(case, {})[m_type] = line
             return
 
         # Mark the file handler as used
@@ -491,6 +499,9 @@ class Command(LAVADaemonCommand):
             if action != "PONG":
                 self.logger.error("Invalid answer '%s'. Should be 'PONG'", action)
                 return
+        except UnicodeDecodeError:
+            self.logger.error("Invalid controler message: can't be decoded")
+            return
         except (IndexError, ValueError):
             self.logger.error("Invalid message '%s'", msg)
             return
