@@ -17,7 +17,6 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
-# pylint: disable=wrong-import-order
 
 import contextlib
 import jinja2
@@ -36,16 +35,18 @@ from django.db import connection, transaction
 from django.db.utils import OperationalError, InterfaceError
 from django.utils import timezone
 
+from lava_common.compat import yaml_dump, yaml_safe_dump, yaml_safe_load
+from lava_common.version import __version__
 from lava_results_app.models import TestCase, TestSuite
+from lava_server.files import File
 from lava_scheduler_app.dbutils import parse_job_description
 from lava_scheduler_app.models import TestJob, Worker
+from lava_scheduler_app.notifications import send_upgraded_master_notifications
 from lava_scheduler_app.scheduler import schedule
-from lava_scheduler_app.utils import mkdir
+from lava_scheduler_app.utils import mkdir, get_encryption_settings
 from lava_server.cmdutils import LAVADaemonCommand, watch_directory
 
 
-# pylint: disable=no-member,bad-continuation
-
 # Current version of the protocol
 # The slave does send the protocol version along with the HELLO and HELLO_RETRY
 # messages. If both version are not identical, the connection is refused by the
@@ -60,11 +61,6 @@ SCHEDULE_INTERVAL = 20
 # Log format
 FORMAT = "%(asctime)-15s %(levelname)7s %(message)s"
 
-# Configuration files
-ENV_PATH = "/etc/lava-server/env.yaml"
-ENV_DUT_PATH = "/etc/lava-server/env.dut.yaml"
-DISPATCHERS_PATH = "/etc/lava-server/dispatcher.d"
-
 
 def send_multipart_u(sock, data):
     """ Wrapper around send_multipart that encode data as bytes.
@@ -75,7 +71,7 @@ def send_multipart_u(sock, data):
     return sock.send_multipart([b(d) for d in data])
 
 
-class SlaveDispatcher:  # pylint: disable=too-few-public-methods
+class SlaveDispatcher:
     def __init__(self, hostname, online=True):
         self.hostname = hostname
         self.last_msg = time.time() if online else 0
@@ -116,26 +112,15 @@ class SlaveDispatcher:  # pylint: disable=too-few-public-methods
                 worker.save()
 
 
-def load_optional_yaml_file(filename, fallback=None):
-    """
-    Returns the string after checking for YAML errors which would cause issues later.
-    Only raise an error if the file exists but is not readable or parsable
-    """
+def config(kind, worker):
     try:
-        with open(filename, "r") as f_in:
-            data_str = f_in.read()
-        yaml.safe_load(data_str)
-        return data_str
-    except FileNotFoundError:
-        # This is ok if the file does not exist
-        if fallback is None:
-            return ""
-        # Use the fallback filename
-        return load_optional_yaml_file(fallback)
+        data = File(kind, worker.hostname).read(raising=False)
+        yaml_safe_load(data)
+        return data
     except yaml.YAMLError:
         # Raise an OSError because the caller uses yaml.YAMLError for a
         # specific usage. Allows here to specify the faulty filename.
-        raise OSError("", "Not a valid YAML file", filename)
+        raise OSError("", f"Invalid YAML file for {worker.hostname}: {kind} file")
 
 
 class Command(LAVADaemonCommand):
@@ -207,14 +192,19 @@ class Command(LAVADaemonCommand):
             send_multipart_u(self.controler, [hostname, "STATUS", str(job.id)])
 
     def dispatcher_alive(self, hostname):
-        if hostname not in self.dispatchers:
-            # The server crashed: send a STATUS message
-            self.logger.warning("Unknown dispatcher <%s> (server crashed)", hostname)
-            self.dispatchers[hostname] = SlaveDispatcher(hostname)
-            self.send_status(hostname)
+        try:
+            if hostname not in self.dispatchers:
+                # The server crashed: send a STATUS message
+                self.logger.warning(
+                    "Unknown dispatcher <%s> (server crashed)", hostname
+                )
+                self.dispatchers[hostname] = SlaveDispatcher(hostname)
+                self.send_status(hostname)
 
-        # Mark the dispatcher as alive
-        self.dispatchers[hostname].alive()
+            # Mark the dispatcher as alive
+            self.dispatchers[hostname].alive()
+        except ValueError:
+            self.logger.error("Invalid dispatcher <%s>", hostname)
 
     def controler_socket(self):
         try:
@@ -232,6 +222,9 @@ class Command(LAVADaemonCommand):
             hostname = u(msg[0])
             # 2: the action
             action = u(msg[1])
+        except UnicodeDecodeError:
+            self.logger.error("Invalid message: can't be decoded")
+            return True
         except (IndexError, ValueError):
             self.logger.error("Invalid message from <%s> '%s'", hostname, msg)
             return True
@@ -244,7 +237,7 @@ class Command(LAVADaemonCommand):
             return True
 
         # Handle the actions
-        if action == "HELLO" or action == "HELLO_RETRY":
+        if action in ["HELLO", "HELLO_RETRY"]:
             self._handle_hello(hostname, action, msg)
         elif action == "PING":
             self._handle_ping(hostname, msg)
@@ -267,6 +260,9 @@ class Command(LAVADaemonCommand):
         try:
             (topic, _, dt, username, data) = (u(m) for m in msg)
             data = simplejson.loads(data)
+        except UnicodeDecodeError:
+            self.logger.error("Invalid event: can't be decoded")
+            return True
         except ValueError:
             self.logger.error("Invalid event: %s", msg)
             return True
@@ -292,8 +288,11 @@ class Command(LAVADaemonCommand):
             job_id = int(msg[2])
             error_msg = u(msg[3])
             compressed_description = msg[4]
+        except UnicodeDecodeError:
+            self.logger.error("Invalid END message: can't be decoded")
+            return
         except (IndexError, ValueError):
-            self.logger.error("Invalid message from <%s> '%s'", hostname, msg)
+            self.logger.error("Invalid END message from <%s> '%s'", hostname, msg)
             return
 
         try:
@@ -352,7 +351,7 @@ class Command(LAVADaemonCommand):
         try:
             slave_version = int(msg[2])
         except (IndexError, ValueError):
-            self.logger.error("Invalid message from <%s> '%s'", hostname, msg)
+            self.logger.error("Invalid HELLO message from <%s> '%s'", hostname, msg)
             return
 
         self.logger.info("%s => %s", hostname, action)
@@ -384,6 +383,16 @@ class Command(LAVADaemonCommand):
         # Mark the dispatcher as alive
         self.dispatcher_alive(hostname)
 
+        # Set the worker version in db.
+        try:
+            Worker.objects.filter(hostname=hostname).update(version=u(msg[3]))
+        except IndexError:
+            self.logger.warning(
+                "HELLO message doesn't contain dispatcher package version <%s> '%s'",
+                hostname,
+                msg,
+            )
+
     def _handle_ping(self, hostname, msg):
         self.logger.debug("%s => PING(%d)", hostname, PING_INTERVAL)
         # Send back a signal
@@ -394,7 +403,7 @@ class Command(LAVADaemonCommand):
         try:
             job_id = int(msg[2])
         except (IndexError, ValueError):
-            self.logger.error("Invalid message from <%s> '%s'", hostname, msg)
+            self.logger.error("Invalid START_OK message from <%s> '%s'", hostname, msg)
             return
         self.logger.info("[%d] %s => START_OK", job_id, hostname)
         try:
@@ -403,25 +412,19 @@ class Command(LAVADaemonCommand):
                 job = TestJob.objects.select_for_update().get(id=job_id)
                 job.go_state_running()
                 job.save()
+            self.dispatcher_alive(hostname)
         except TestJob.DoesNotExist:
             self.logger.error("[%d] Unknown job", job_id)
-        else:
-            self.dispatcher_alive(hostname)
 
-    def export_definition(self, job):  # pylint: disable=no-self-use
-        job_def = yaml.safe_load(job.definition)
-        job_def["compatibility"] = job.pipeline_compatibility
-
-        # no need for the dispatcher to retain comments
-        return yaml.dump(job_def)
-
-    def save_job_config(self, job, device_cfg, env_str, env_dut_str, dispatcher_cfg):
+    def save_job_config(
+        self, job, job_def_str, device_cfg, env_str, env_dut_str, dispatcher_cfg
+    ):
         output_dir = job.output_dir
         mkdir(output_dir)
         with open(os.path.join(output_dir, "job.yaml"), "w") as f_out:
-            f_out.write(self.export_definition(job))
+            f_out.write(job_def_str)
         with open(os.path.join(output_dir, "device.yaml"), "w") as f_out:
-            yaml.dump(device_cfg, f_out)
+            f_out.write(device_cfg)
         if env_str:
             with open(os.path.join(output_dir, "env.yaml"), "w") as f_out:
                 f_out.write(env_str)
@@ -435,30 +438,24 @@ class Command(LAVADaemonCommand):
     def start_job(self, job):
         # Load job definition to get the variables for template
         # rendering
-        job_def = yaml.safe_load(job.definition)
+        job_def = yaml_safe_load(job.definition)
+        job_def["compatibility"] = job.pipeline_compatibility
+        job_def_str = yaml_safe_dump(job_def)
         job_ctx = job_def.get("context", {})
 
         device = job.actual_device
         worker = device.worker_host
 
         # TODO: check that device_cfg is not None!
-        device_cfg = device.load_configuration(job_ctx)
+        device_cfg_str = device.load_configuration(job_ctx, output_format="yaml")
 
-        # Try to load the dispatcher specific files and then fallback to the
-        # default configuration files.
-        env_str = load_optional_yaml_file(
-            os.path.join(DISPATCHERS_PATH, worker.hostname, "env.yaml"), ENV_PATH
-        )
-        env_dut_str = load_optional_yaml_file(
-            os.path.join(DISPATCHERS_PATH, worker.hostname, "env.dut.yaml"),
-            ENV_DUT_PATH,
-        )
-        dispatcher_cfg = load_optional_yaml_file(
-            os.path.join(DISPATCHERS_PATH, worker.hostname, "dispatcher.yaml"),
-            os.path.join(DISPATCHERS_PATH, "%s.yaml" % worker.hostname),
-        )
+        env_str = config("env", worker)
+        env_dut_str = config("env-dut", worker)
+        dispatcher_cfg = config("dispatcher", worker)
 
-        self.save_job_config(job, device_cfg, env_str, env_dut_str, dispatcher_cfg)
+        self.save_job_config(
+            job, job_def_str, device_cfg_str, env_str, env_dut_str, dispatcher_cfg
+        )
         self.logger.info(
             "[%d] START => %s (%s)", job.id, worker.hostname, device.hostname
         )
@@ -468,28 +465,42 @@ class Command(LAVADaemonCommand):
                 worker.hostname,
                 "START",
                 str(job.id),
-                self.export_definition(job),
-                yaml.dump(device_cfg),
+                job_def_str,
+                device_cfg_str,
                 dispatcher_cfg,
                 env_str,
                 env_dut_str,
             ],
         )
 
+        if not job.is_multinode:
+            return
+
+        device_cfg = yaml_safe_load(device_cfg_str)
         # For multinode jobs, start the dynamic connections
-        parent = job
         for sub_job in job.sub_jobs_list:
-            if sub_job == parent or not sub_job.dynamic_connection:
+            if sub_job == job or not sub_job.dynamic_connection:
                 continue
 
+            # Render the sub job definition
+            sub_job_def = yaml_safe_load(sub_job.definition)
+            sub_job_def["compatibility"] = sub_job.pipeline_compatibility
+            sub_job_def_str = yaml_safe_dump(sub_job_def)
+
             # inherit only enough configuration for dynamic_connection operation
             self.logger.info(
                 "[%d] Trimming dynamic connection device configuration.", sub_job.id
             )
-            min_device_cfg = parent.actual_device.minimise_configuration(device_cfg)
+            min_device_cfg = job.actual_device.minimise_configuration(device_cfg)
+            min_device_cfg_str = yaml_safe_dump(min_device_cfg)
 
             self.save_job_config(
-                sub_job, min_device_cfg, env_str, env_dut_str, dispatcher_cfg
+                sub_job,
+                sub_job_def_str,
+                min_device_cfg_str,
+                env_str,
+                env_dut_str,
+                dispatcher_cfg,
             )
             self.logger.info(
                 "[%d] START => %s (connection)", sub_job.id, worker.hostname
@@ -500,8 +511,8 @@ class Command(LAVADaemonCommand):
                     worker.hostname,
                     "START",
                     str(sub_job.id),
-                    self.export_definition(sub_job),
-                    yaml.dump(min_device_cfg),
+                    sub_job_def_str,
+                    min_device_cfg_str,
                     dispatcher_cfg,
                     env_str,
                     env_dut_str,
@@ -571,7 +582,7 @@ class Command(LAVADaemonCommand):
                     name="job",
                     suite=suite,
                     result=TestCase.RESULT_FAIL,
-                    metadata=yaml.dump(metadata),
+                    metadata=yaml_safe_dump(metadata),
                 )
                 job.go_state_finished(TestJob.HEALTH_INCOMPLETE, True)
                 job.save()
@@ -602,6 +613,10 @@ class Command(LAVADaemonCommand):
         # Initialize logging.
         self.setup_logging("lava-master", options["level"], options["log_file"], FORMAT)
 
+        self.logger.info("[INIT] Starting lava-master")
+        self.logger.info("[INIT] Version %s", __version__)
+        self.logger.info("[INIT] Using protocol version %d", PROTOCOL_VERSION)
+
         self.logger.info("[INIT] Dropping privileges")
         if not self.drop_privileges(options["user"], options["group"]):
             self.logger.error("[INIT] Unable to drop privileges")
@@ -610,7 +625,7 @@ class Command(LAVADaemonCommand):
         filename = os.path.join(settings.MEDIA_ROOT, "lava-master-config.yaml")
         self.logger.debug("[INIT] Dumping config to %s", filename)
         with open(filename, "w") as output:
-            yaml.dump(options, output)
+            yaml_dump(options, output)
 
         self.logger.info("[INIT] Marking all workers as offline")
         with transaction.atomic():
@@ -628,21 +643,26 @@ class Command(LAVADaemonCommand):
             self.controler.setsockopt(zmq.IPV6, 1)
             self.event_socket.setsockopt(zmq.IPV6, 1)
 
-        if options["encrypt"]:
+        encryption_settings = get_encryption_settings(options)
+        if encryption_settings["encrypt"]:
             self.logger.info("[INIT] Starting encryption")
             try:
                 self.auth = ThreadAuthenticator(context)
                 self.auth.start()
                 self.logger.debug(
-                    "[INIT] Opening master certificate: %s", options["master_cert"]
+                    "[INIT] Opening master certificate: %s",
+                    encryption_settings["master_cert"],
                 )
                 master_public, master_secret = zmq.auth.load_certificate(
-                    options["master_cert"]
+                    encryption_settings["master_cert"]
                 )
                 self.logger.debug(
-                    "[INIT] Using slaves certificates from: %s", options["slaves_certs"]
+                    "[INIT] Using slaves certificates from: %s",
+                    encryption_settings["slaves_certs"],
+                )
+                self.auth.configure_curve(
+                    domain="*", location=encryption_settings["slaves_certs"]
                 )
-                self.auth.configure_curve(domain="*", location=options["slaves_certs"])
             except OSError as err:
                 self.logger.error(err)
                 self.auth.stop()
@@ -651,8 +671,8 @@ class Command(LAVADaemonCommand):
             self.controler.curve_secretkey = master_secret
             self.controler.curve_server = True
 
-            self.logger.debug("[INIT] Watching %s", options["slaves_certs"])
-            self.inotify_fd = watch_directory(options["slaves_certs"])
+            self.logger.debug("[INIT] Watching %s", encryption_settings["slaves_certs"])
+            self.inotify_fd = watch_directory(encryption_settings["slaves_certs"])
             if self.inotify_fd is None:
                 self.logger.error("[INIT] Unable to start inotify")
 
@@ -680,9 +700,11 @@ class Command(LAVADaemonCommand):
         (self.pipe_r, _) = self.setup_zmq_signal_handler()
         self.poller.register(self.pipe_r, zmq.POLLIN)
 
-        self.logger.info("[INIT] LAVA master has started.")
-        self.logger.info("[INIT] Using protocol version %d", PROTOCOL_VERSION)
+        # Send master upgrade notifications.
+        send_upgraded_master_notifications(__version__, self.logger)
 
+        # Main loop
+        self.logger.info("[INIT] Starting main loop")
         try:
             self.main_loop(options)
         except BaseException as exc:
@@ -695,12 +717,13 @@ class Command(LAVADaemonCommand):
             )
             self.controler.close(linger=0)
             self.event_socket.close(linger=0)
-            if options["encrypt"]:
+            if encryption_settings["encrypt"]:
                 self.auth.stop()
             context.term()
 
     def main_loop(self, options):
         last_schedule = last_dispatcher_check = time.time()
+        encryption_settings = get_encryption_settings(options)
 
         while True:
             try:
@@ -747,12 +770,19 @@ class Command(LAVADaemonCommand):
                 # Inotify socket
                 if sockets.get(self.inotify_fd) == zmq.POLLIN:
                     os.read(self.inotify_fd, 4096)
-                    self.logger.info(
-                        "[AUTH] Reloading certificates from %s", options["slaves_certs"]
-                    )
-                    self.auth.configure_curve(
-                        domain="*", location=options["slaves_certs"]
-                    )
+                    if self.auth is not None:
+                        self.logger.info(
+                            "[AUTH] Reloading certificates from %s",
+                            encryption_settings["slaves_certs"],
+                        )
+                        self.auth.configure_curve(
+                            domain="*", location=encryption_settings["slaves_certs"]
+                        )
+                    else:
+                        self.logger.error(
+                            "[AUTH] New certificates in %s but encryption is disabled",
+                            encryption_settings["slaves_certs"],
+                        )
 
                 # Check dispatchers status
                 now = time.time()
