@@ -23,6 +23,7 @@ from zmq.utils.strtypes import u
 
 from django.conf import settings
 
+from lava_common.version import __version__
 from lava_server.cmdutils import LAVADaemonCommand
 
 
@@ -72,6 +73,9 @@ class Command(LAVADaemonCommand):
             "lava-publisher", options["level"], options["log_file"], FORMAT
         )
 
+        self.logger.info("[INIT] Starting lava-publisher")
+        self.logger.info("[INIT] Version %s", __version__)
+
         self.logger.info("[INIT] Dropping privileges")
         if not self.drop_privileges(options["user"], options["group"]):
             self.logger.error("[INIT] Unable to drop privileges")
