@@ -97,7 +97,7 @@ class AuthToken(models.Model):
         null=True, help_text="Time and date when the token was last used"
     )
 
-    user = models.ForeignKey(User, related_name="auth_tokens")
+    user = models.ForeignKey(User, related_name="auth_tokens", on_delete=models.CASCADE)
 
     def __str__(self):
         return u"security token {pk}".format(pk=self.pk)
@@ -110,15 +110,12 @@ class AuthToken(models.Model):
         This also bumps last_used_on if successful
         """
         try:
-            token = cls.objects.get(secret=secret)
-        except cls.DoesNotExist:
-            return None
-        else:
-            if token.user.username != username:
-                return None  # bad username for this secret
+            token = cls.objects.get(user__username=username, secret=secret)
             token.last_used_on = timezone.now()
             token.save()
             return token.user
+        except cls.DoesNotExist:
+            return None
 
 
 def xml_rpc_signature(*sig):
@@ -175,10 +172,9 @@ class CallContext:
     """
 
     def __init__(self, user, mapper, dispatcher, request=None):
-        if user is not None and user.is_authenticated() and user.is_active:
+        self.user = None
+        if user is not None and user.is_authenticated and user.is_active:
             self.user = user
-        else:
-            self.user = None
         self.mapper = mapper
         self.dispatcher = dispatcher
         self.request = request
@@ -594,7 +590,7 @@ class SystemAPI(ExposedAPI):
                 FaultCodes.ServerError.INVALID_METHOD_PARAMETERS,
                 "system.multicall params must be an array",
             )
-        if len(subcall) > 0:
+        if subcall:
             return xmlrpc.client.Fault(
                 FaultCodes.ServerError.INVALID_METHOD_PARAMETERS,
                 "system.multicall specified additional arguments %s"
