@@ -3,7 +3,7 @@
 #
 #  requires.py
 #
-#  Copyright 2018 Linaro
+#  Copyright 2018-2019 Linaro Limited
 #  Author: Neil Williams <neil.williams@linaro.org>
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -25,16 +25,13 @@ import sys
 import yaml
 import argparse
 
-
-"""
-Goals:
-0: Convert a packaged list of dependencies and versions
-   to a distro-specific list of dependencies.
-1: Remove need for requirements.txt using Pip syntax as this is
-   misleading.
-2: output a list of binary package names for the requested distribution
-   and suite to pass to docker scripts and LXC unit test jobs.
-"""
+#  Goals:
+#  0: Convert a packaged list of dependencies and versions
+#     to a distro-specific list of dependencies.
+#  1: Remove need for requirements.txt using Pip syntax as this is
+#     misleading.
+#  2: output a list of binary package names for the requested distribution
+#     and suite to pass to docker scripts and LXC unit test jobs.
 
 
 def debian(args, depends):
@@ -48,26 +45,26 @@ def debian(args, depends):
     calls, 1 for backports and one for the parent.
     """
     if args.unittests:
-        unittests = []
+        unittests = set({})
         if args.names and depends[args.package]:
             for key, item in depends[args.package].items():
                 if depends[args.package][key].get("unittests"):
-                    unittests.append(item["name"])
+                    unittests.add(item["name"])
             if unittests:
                 print(" ".join(sorted(unittests)))
         return 0
     if args.names:
-        msg = []
-        backports = []
+        msg = set({})
+        backports = set({})
         if not depends.get(args.package):
             return 0
         for key, item in depends[args.package].items():
             if depends[args.package][key].get("unittests"):
                 continue
             if args.suite.endswith("-backports"):
-                backports.append(item["name"])
+                backports.add(item["name"])
                 continue
-            msg.append(item["name"])
+            msg.add(item["name"])
         if backports:
             print(" ".join(sorted(backports)))
         elif msg:
