@@ -20,12 +20,12 @@
 # along with this program; if not, see <http://www.gnu.org/licenses>.
 
 from datetime import timedelta
+import logging
 
 from django.contrib.auth.models import User
 from django.test import TestCase
 from django.utils import timezone
 
-from tests.utils import DummyLogger
 from lava_scheduler_app.models import Device, DeviceType, TestJob, Worker
 from lava_scheduler_app.scheduler import schedule, schedule_health_checks
 
@@ -127,7 +127,7 @@ class TestHealthCheckScheduling(TestCase):
         self.assertEqual(self.device02.get_health_check(), None)
         self.assertEqual(self.device03.get_health_check(), None)
         # Schedule without health check
-        available_devices = schedule_health_checks(DummyLogger())[0]
+        available_devices = schedule_health_checks(logging.getLogger())[0]
         self.assertEqual(available_devices, {"panda": ["panda01", "panda03"]})
 
     def test_disabled_hc(self):
@@ -139,7 +139,7 @@ class TestHealthCheckScheduling(TestCase):
 
         self.device_type01.disable_health_check = True
         self.device_type01.save()
-        available_devices = schedule_health_checks(DummyLogger())[0]
+        available_devices = schedule_health_checks(logging.getLogger())[0]
         self.assertEqual(available_devices, {"panda": ["panda01", "panda03"]})
 
     def test_no_devicedict(self):
@@ -152,7 +152,7 @@ class TestHealthCheckScheduling(TestCase):
         self.device04.health = Device.HEALTH_UNKNOWN
         self.device04.save()
 
-        schedule_health_checks(DummyLogger())
+        schedule_health_checks(logging.getLogger())
 
         self.device04.refresh_from_db()
         self.assertFalse(self.device04.is_valid())
@@ -166,7 +166,7 @@ class TestHealthCheckScheduling(TestCase):
         self.assertNotEqual(self.device02.get_health_check(), None)
         self.assertNotEqual(self.device03.get_health_check(), None)
 
-        available_devices = schedule_health_checks(DummyLogger())[0]
+        available_devices = schedule_health_checks(logging.getLogger())[0]
         self.assertEqual(available_devices, {"panda": []})
         self._check_hc_scheduled(self.device01)
         self._check_hc_not_scheduled(self.device02)
@@ -185,7 +185,7 @@ class TestHealthCheckScheduling(TestCase):
         self.device02.save()
         self.device03.health = Device.HEALTH_GOOD
         self.device03.save()
-        available_devices = schedule_health_checks(DummyLogger())[0]
+        available_devices = schedule_health_checks(logging.getLogger())[0]
         self.assertEqual(available_devices, {"panda": ["panda03"]})
         self._check_hc_scheduled(self.device01)
         self._check_hc_not_scheduled(self.device02)
@@ -204,7 +204,7 @@ class TestHealthCheckScheduling(TestCase):
         self.device02.save()
         self.device03.health = Device.HEALTH_LOOPING
         self.device03.save()
-        available_devices = schedule_health_checks(DummyLogger())[0]
+        available_devices = schedule_health_checks(logging.getLogger())[0]
         self.assertEqual(available_devices, {"panda": []})
         self._check_hc_scheduled(self.device01)
         self._check_hc_not_scheduled(self.device02)
@@ -229,7 +229,7 @@ class TestHealthCheckScheduling(TestCase):
             self.device02.save()
             self.device03.health = health
             self.device03.save()
-            available_devices = schedule_health_checks(DummyLogger())[0]
+            available_devices = schedule_health_checks(logging.getLogger())[0]
             self.assertEqual(available_devices, {"panda": []})
             self._check_hc_not_scheduled(self.device01)
             self._check_hc_not_scheduled(self.device02)
@@ -257,7 +257,7 @@ class TestHealthCheckScheduling(TestCase):
             submitter=self.user,
             definition=_minimal_valid_job(None),
         )
-        schedule(DummyLogger())
+        schedule(logging.getLogger())
         self.device01.refresh_from_db()
         j.refresh_from_db()
         self.assertEqual(j.state, TestJob.STATE_SCHEDULED)
@@ -275,7 +275,7 @@ class TestHealthCheckScheduling(TestCase):
         self.last_hc03.submit_time = timezone.now() - timedelta(hours=25)
         self.last_hc03.save()
 
-        schedule(DummyLogger())
+        schedule(logging.getLogger())
         self.device03.refresh_from_db()
         j.refresh_from_db()
         self.assertEqual(j.state, TestJob.STATE_SUBMITTED)
@@ -316,7 +316,7 @@ class TestHealthCheckScheduling(TestCase):
             definition=_minimal_valid_job(None),
         )
 
-        schedule(DummyLogger())
+        schedule(logging.getLogger())
         self.device03.refresh_from_db()
         j01.refresh_from_db()
         self.assertEqual(j01.state, TestJob.STATE_SCHEDULED)
@@ -325,7 +325,7 @@ class TestHealthCheckScheduling(TestCase):
         j01.start_time = timezone.now() - timedelta(hours=1)
         j01.save()
 
-        schedule(DummyLogger())
+        schedule(logging.getLogger())
         self.device03.refresh_from_db()
         j02.refresh_from_db()
         self.assertEqual(j02.state, TestJob.STATE_SCHEDULED)
@@ -334,7 +334,7 @@ class TestHealthCheckScheduling(TestCase):
         j02.start_time = timezone.now() - timedelta(hours=1)
         j02.save()
 
-        schedule(DummyLogger())
+        schedule(logging.getLogger())
         self.device03.refresh_from_db()
         j03.refresh_from_db()
         self.assertEqual(j03.state, TestJob.STATE_SUBMITTED)
@@ -430,7 +430,7 @@ actions: []
         self.device_type01.disable_health_check = False
         self.device_type01.save()
 
-        schedule_health_checks(DummyLogger())[0]
+        schedule_health_checks(logging.getLogger())[0]
 
         self._check_hc_scheduled(self.device01)
         self._check_hc_not_scheduled(self.device02)
@@ -442,7 +442,7 @@ actions: []
         self.device_type01.disable_health_check = False
         self.device_type01.save()
 
-        schedule_health_checks(DummyLogger())[0]
+        schedule_health_checks(logging.getLogger())[0]
 
         self._check_hc_scheduled(self.device01)
         self._check_hc_not_scheduled(self.device02)
@@ -455,7 +455,7 @@ actions: []
         self.device_type01.disable_health_check = False
         self.device_type01.save()
 
-        schedule_health_checks(DummyLogger())[0]
+        schedule_health_checks(logging.getLogger())[0]
 
         self._check_hc_scheduled(self.device01)
         self._check_hc_not_scheduled(self.device02)
@@ -505,7 +505,7 @@ class TestPriorities(TestCase):
             )
             jobs.append(j)
 
-        log = DummyLogger()
+        log = logging.getLogger()
         schedule(log)
         self.device01.refresh_from_db()
         self.assertEqual(self.device01.state, Device.STATE_RESERVED)
@@ -611,7 +611,7 @@ class TestPriorities(TestCase):
             jobs.append(j)
 
         # Check that an health check will be scheduled before any jobs
-        log = DummyLogger()
+        log = logging.getLogger()
         schedule(log)
         self.device01.refresh_from_db()
         self.assertEqual(self.device01.state, Device.STATE_RESERVED)
@@ -690,7 +690,7 @@ class TestJobLimitHc1(TestCase):
 
     def test_job_limit_hc(self):
 
-        schedule_health_checks(DummyLogger())
+        schedule_health_checks(logging.getLogger())
 
         devs = 0
         # check that only one device got healthcheck
@@ -709,7 +709,7 @@ class TestJobLimitHc1(TestCase):
             job.save()
 
         # STEP 2
-        schedule_health_checks(DummyLogger())
+        schedule_health_checks(logging.getLogger())
 
         devs = 0
         for device in self.devices:
@@ -768,7 +768,7 @@ class TestJobLimitHc2(TestCase):
 
     def test_job_limit_hc2(self):
 
-        schedule_health_checks(DummyLogger())
+        schedule_health_checks(logging.getLogger())
 
         devs = 0
         # check that only 2 devices got healthcheck
@@ -788,7 +788,7 @@ class TestJobLimitHc2(TestCase):
             job.save()
 
         # STEP 2
-        schedule_health_checks(DummyLogger())
+        schedule_health_checks(logging.getLogger())
 
         devs = 0
         # check that only 4 devices got healthcheck
@@ -802,80 +802,51 @@ class TestJobLimitHc2(TestCase):
 # test both healthcheck and normal testjobs with joblimit
 class TestJobLimit(TestCase):
     def setUp(self):
-        self.joblimit = 2
-        self.devmax = 6
+        self.logger = logging.getLogger()
+        self.logger.setLevel(logging.DEBUG)
+        self.job_limit = 2
 
         self.worker01 = Worker.objects.create(
-            hostname="worker-01", state=Worker.STATE_ONLINE, job_limit=2
+            hostname="worker-01", state=Worker.STATE_ONLINE, job_limit=self.job_limit
         )
         self.user = User.objects.create(username="user-01")
-        self.device_type01 = DeviceType.objects.create(name="qemu")
+        self.device_type01 = DeviceType.objects.create(
+            name="qemu", disable_health_check=True
+        )
         self.devices = []
-        j = self.devmax
-        while j > 0:
+        for i in range(0, 6):
             dev = Device.objects.create(
-                hostname="qemu0%d" % j,
+                hostname=f"qemu0{i}",
                 device_type=self.device_type01,
                 worker_host=self.worker01,
                 health=Device.HEALTH_GOOD,
             )
-            dev.save()
             self.devices.append(dev)
-            j = j - 1
-
-        self.original_health_check = Device.get_health_check
-        Device.get_health_check = _minimal_valid_job
-
-    def tearDown(self):
-        Device.get_health_check = self.original_health_check
 
     def test_job_limit(self):
-
-        self.jobs = []
-        maxjobs = 8
-        j = maxjobs
-        while j > 0:
-            j = j - 1
-            job = TestJob.objects.create(
+        for i in range(0, 4):
+            TestJob.objects.create(
                 requested_device_type=self.device_type01,
                 submitter=self.user,
                 definition=_minimal_valid_job(None),
             )
-            self.jobs.append(job)
+        assert TestJob.objects.all().count() == 4
+        # Limit the number of jobs that can run
+        schedule(self.logger)
+        assert TestJob.objects.filter(state=TestJob.STATE_SCHEDULED).count() == 2
+        assert TestJob.objects.filter(state=TestJob.STATE_SUBMITTED).count() == 2
 
-        j = self.devmax
-        while j > 0:
-            j = j - 2
-            djobs = schedule(DummyLogger())
-            self.assertEqual(len(djobs), self.joblimit)
-            for job in TestJob.objects.filter(
-                state__in=[TestJob.STATE_SCHEDULING, TestJob.STATE_SCHEDULED]
-            ):
-                self.assertTrue(job.health_check)
-                job.go_state_finished(TestJob.HEALTH_COMPLETE)
-                job.actual_device.health = Device.HEALTH_GOOD
-                job.actual_device.state = Device.STATE_IDLE
-                job.actual_device.save()
-                job.save()
-
-        j = maxjobs
-        while j > 0:
-            j = j - 2
-            djobs = schedule(DummyLogger())
-            self.assertEqual(len(djobs), self.joblimit)
-            devs = 0
-            for device in self.devices:
-                device.refresh_from_db()
-                if device.state != Device.STATE_IDLE:
-                    devs = devs + 1
-            self.assertEqual(devs, self.joblimit)
-
-            for job in TestJob.objects.filter(
-                state__in=[TestJob.STATE_SCHEDULING, TestJob.STATE_SCHEDULED]
-            ):
-                self.assertFalse(job.health_check)
-                job.go_state_finished(TestJob.HEALTH_COMPLETE)
-                job.actual_device.health = Device.HEALTH_GOOD
-                job.actual_device.state = Device.STATE_IDLE
-                job.actual_device.save()
-                job.save()
+    def test_job_limit_unlimited(self):
+        for i in range(0, 4):
+            TestJob.objects.create(
+                requested_device_type=self.device_type01,
+                submitter=self.user,
+                definition=_minimal_valid_job(None),
+            )
+        assert TestJob.objects.all().count() == 4
+        # Limit the number of jobs that can run
+        self.worker01.job_limit = 0
+        self.worker01.save()
+        schedule(self.logger)
+        assert TestJob.objects.filter(state=TestJob.STATE_SCHEDULED).count() == 4
+        assert TestJob.objects.filter(state=TestJob.STATE_SUBMITTED).count() == 0
